/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.api;

/** The console's imports */
import org.objectweb.util.browser.api.Context;

import javax.swing.JTextField;

/**
 * Interface which allows to configure the dynamic tree
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public interface TreeConfiguration {

    /** 
     * Creates a new browser property with the given fileName.
     * @param files The properties files to parse and to apply
     */
    public void setNewBrowserProperty(String[] files);

    /** 
     * Sets a new browser property.
     * @param properties The properties to apply
     */
    public void setNewBrowserProperty(BrowserProperty properties);

    /**
     * Adds the new properties
     * @param file The properties files to add
     */
    public void addBrowserProperty(String[] file);

    /**
     * Gives the current BrowserProperty. 
     * @return The current BrowserProperty
     */
    public BrowserProperty getBrowserProperty();

    /**
     * Creates a new context property with the given fileName
     * @param file The properties files to parse and to apply
     */
    public void setNewContextProperty(String file);

    /**
     * Sets a new context property
     * @param properties The properties to apply 
     */
    public void setNewContextProperty(ContextProperty properties);

    /**
     * Gives the current ContextProperty.
     * @return The current ContextProperty
     */
    public ContextProperty getContextProperty();

    /**
     * Gives the initial context 
     * @return The initial context
     */
    public Context getInitialContext();

    /**
     * Returns true if this initial context contains an entry for the specified id.
     * @param id Key whose presence in this initial context is to be tested.
     * @return True if this initial context contains an entry for the specified id.
     */
    public boolean containsEntry(String id);

    /**
     * Fixes the target panel for the tree (Where the associated panels will be displayed).
     * @param targetPanel The panel to use to display associated panels
     */
    public void setTargetPanel(TargetPanel targetPanel);

    /**
     * Fixes the status bar for the tree (Where info will be displayed)
     * @param statusField
     */
    public void setStatusField(JTextField statusField);

    /**
     * Indicates if the popup are enabled or not. By default, the popup menus are enabled.
     * @param popupEnabled True if the popup menu have to be enabled, false either.
     */
    public void setPopupEnabled(boolean popupEnabled);
    
    /**
     * Returns true if the popup menus are enabled.
     * @return true if the popup menus are enabled.
     */
    public boolean isPopupEnabled();
    
}
