/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.core.common;

import org.objectweb.util.browser.core.api.TypeKey;

/**
 * Basic implementation of TypeKey interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 * 
 * @version 0.1
 */
public class DefaultTypeKey 
    implements TypeKey 
{

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The type system of the key. */
    protected String typeSystem_;
    
    /** The type name of the key. */
    protected String typeName_;

    //==================================================================
    //
    // Constructors.
    // 
    //==================================================================

    /**
     * Default constructor
     * @param typeSystem The type system of the key.
     * @param typeName The type name of the key.
     */
    public DefaultTypeKey(String typeSystem, String typeName)
    {
        typeSystem_ = typeSystem;
        typeName_ = typeName;
    }

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================

    //==================================================================
    //
    // Public methods for TypeKey interface.
    //
    //==================================================================

    /**
     * Provides the type system of the key.
     * @return The type system of the key.
     */
    public String
    getTypeSystem()
    {
        return typeSystem_;    
    }
    
    /**
     * Provides the type name of the key.
     * @return The type name of the key.
     */
    public String
    getTypeName()
    {
        return typeName_;
    }


}