/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.naming;

import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Wrapper;

/**
 * Defines some methods used by all context
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public abstract class AbstractContext 
    implements Context, Wrapper {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Creates an Entry.
     *
     * @param value The value of the entry to create.
     * @param id The id of the name of the entry to create.
     *
     * @return A new Entry.
     *
     * postcondition:<br>
     *              return.getOWValue() == value
     * and return.getOWName().getOWID() == id
     * and        return.getOWContext() == this;
     */
    protected Entry createEntry(Object value, String id) {
        return new DefaultEntry(value, new DefaultName(id), this);
    }

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    /**
     * Returns an array containing the entries contained by the target context.
     * @return A new array of entry.
     */
    public abstract Entry[] getEntries();

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fixes the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public abstract void setWrapped(Object object);

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public abstract Object getWrapped();

}
