/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.naming;

/** To use the Name interface. */
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.core.api.ExtendedEntry;
import org.objectweb.util.browser.core.api.Name;

/** To use the Context interface. */

/** To use the ExtendedEntry interface */

/**
 * Default basic implementation of the Entry interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */
public class DefaultEntry extends AbstractEntry implements ExtendedEntry {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The value. */
    private Object value_;

    /** The name. */
    private Name name_;

    /** The context. */
    private Context context_;

    /** The wrapped object */
    private Object wrappedObject_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Constructs a DefaultEntry with a value, name, and context.
     *
     * This constructor sets the value, name and context
     * to respectively the given value, name, and context.
     *
     * @param value The value.
     * @param name The name.
     * @param context The context.
     *
     * @postcondition
     *        getOWValue() == value
     *  and    getOWName() == name
     *  and getOWContext() == context;
     */
    public DefaultEntry(Object value, Name name, Context context) {
        // Inits the internal state.
        value_ = value;
        name_ = name;
        context_ = context;
        wrappedObject_ = value;
    }

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for class Object.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface Entry.
    //
    // ==================================================================

    /**
     * Gets the value contained in the target entry.
     *
     * This method does not update the value contained in the target entry.
     *
     * @return The value contained in the target entry.
     *
     * @postcondition return == getOWValue();
     */
    public Object getValue() {
        return value_;
    }

    /**
     * Gets the name of the target entry, in the context it belongs to.
     *
     * This method does not update the name of the target entry.
     *
     * @return The name of the target entry, in the context it belongs to.
     *
     * @postcondition return == getOWName();
     */
    public Object getName() {
        return name_.getOWId();
    }

    /**
     * Gets the context containing the target entry,
     * if any, null otherwise.
     *
     * This method does not update the context containing
     * the target entry.
     *
     * @return The context containing the target entry,
     *         if any, null otherwise..
     *
     * @postcondition return == getOWContext();
     */
    public Context getContext() {
        return context_;
    }

    /**
     * Changes the value of the Entry.
     *
     * @param value The value to set.
     */
    public void setOWValue(Object value) {
        value_ = value;
    }

    /**
     * Returns the wrapped object. The returned value is the same as the getOWObject method if no Context is defined for this object.
     *
     * @return The wrapped object
     */
    public Object getWrappedObject() {
        return wrappedObject_;
    }

    /**
     * Sets the wrapped object. It's necessary when the object crosses more than one factory, as every CORBA Object does.
     */
    public void setWrappedObject(Object wrapped) {
        wrappedObject_ = wrapped;
    }

}
