/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.popup;

/**
 * Contains the data of one item of a popup menu.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class ItemProperty {
	
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The label of the item. */
    protected String label_;

    /** The name of the class containing the action. */
    protected String className_;

    /** Specifies if this item must be visible by children at the tree level. */
    protected boolean treeChildVisible_;

    /** Specifies if this item must be visible by children at the type level. */
    protected boolean typeChildVisible_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor.
     * Equivalent to ItemProperty (label,className,false,true).
     *
     * @param label The label of the item
     * @param className The name of the class containing the action
     */
    public ItemProperty(String label, String className) {
        this(label, className, false, true);
    }

    /**
     * Default constructor
     * @param label The label of the item
     * @param className The name of the class containing the action
     * @param treeChildVisible Specifies if this item must be visible by children at the tree level 
     * @param typeChildVisible Specifies if this item must be visible by children at the type level
     */
    public ItemProperty(String label, String className, boolean treeChildVisible, boolean typeChildVisible) {
        label_ = label;
        className_ = className;
        treeChildVisible_ = treeChildVisible;
        typeChildVisible_ = typeChildVisible;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /** 
     * Returns the label of the item.
     * @return The label of the item
     */
    public String getLabel() {
        return label_;
    }

    /**
     * Returns the name of the class containing the action.
     * @return The name of the class to instanciate.
     */
    public String getClassName() {
        return className_;
    }

    /**
     * Specifies if this item must be visible by children at the tree level.
     * @return true if this item must be visible by children at the tree level
     */
    public boolean isTreeChildVisible() {
        return treeChildVisible_;
    }

    /**
     * Specifies if this item must be visible by children at the type level.
     * @return true if this item must be visible by children at the type level
     */
    public boolean isTypeChildVisible() {
        return typeChildVisible_;
    }
}
