/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.plugins.java;

/** To use the AbstractContext class. */
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.naming.AbstractContext;

/** To use the java.util.Properties interface. */
import java.util.Enumeration;
import java.util.Properties;

/**
 * Basic implementation of the Context interface
 * on top of a java.util.Properties instance.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */
public class JavaPropertiesContext 
    extends AbstractContext {
	
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The properties. */
    private Properties properties_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    /**
     * Returns an array containing the entries contained by the target context.
     * @return A new array of entry.
     */
    public Entry[] getEntries(){
        Entry[] entries = new Entry[properties_.size()];
        Enumeration keys, elements;
        keys = properties_.keys();
        elements = properties_.elements();
        for (int i = 0; i < entries.length; i++) {
			entries[i] = createEntry(elements.nextElement(),(String) keys.nextElement()); 
		}
        return entries;
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public void setWrapped(Object object) {
        properties_ = (Properties) object;
    }

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public Object getWrapped() {
        return properties_;
    }

}
