/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package test;

/** The administration console's API*/
import org.objectweb.util.browser.api.Tree;
import org.objectweb.util.browser.core.api.ContextContainer;
import org.objectweb.util.browser.core.naming.DefaultContextContainer;
import org.objectweb.util.browser.core.common.DefaultTreePanel;
import org.objectweb.util.browser.core.common.DefaultViewPanel;
import org.objectweb.util.browser.core.common.DynamicTree;

/** The java API's imports */
import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JSplitPane;
import javax.swing.JTextField;


/**
 * This class defines the main frame
 * 
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class Main extends JFrame{

    /**
     * Populates the tree
     */
    protected void populateTree(Tree tree) {

        tree.addEntry("Java Properties", System.getProperties());

        Calculate c = new Calculate();
    
        ContextContainer contextContainer_ = new DefaultContextContainer();        
        contextContainer_.addEntry("Calculate", c);
        tree.addEntry("Test", contextContainer_, 1);
        tree.addEntry("Object", new Object(), 1);
        tree.addEntry("Calculatrice",new Calculate());
    }

    /**
     * Default constructor
     */
    public Main(String [] browserPropertyFile) {

        super("Browser GUI");

        DefaultViewPanel viewPanel = new DefaultViewPanel();

        DynamicTree tree = new DynamicTree();
        tree.setNewBrowserProperty(browserPropertyFile);
        populateTree((Tree)tree);
        tree.setTargetPanel(viewPanel);

        JTextField statusBar = new JTextField();
        statusBar.setBorder(BorderFactory.createLoweredBevelBorder());
        statusBar.setEditable(false);
        tree.setStatusField(statusBar);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,true,new DefaultTreePanel(tree),viewPanel),BorderLayout.CENTER);
        getContentPane().add(statusBar,BorderLayout.SOUTH);
        
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        pack();

        // Centers the frame
        Dimension screenSize =
            java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screenSize.width - getWidth()) / 2,(screenSize.height - getHeight()) / 2);

        setVisible(true);
    }

    public static void main(String[] args) {
        java.util.List l = new java.util.Vector();
        l.add("test/resources/config/commonConfig.xml");
        l.add("test/resources/config/demoConfig.xml");
        new Main((String [])l.toArray(new String[0]));
    }

}



