/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.pss.demo1;

public class Server
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * The application entry point
     */
    public static void main( String [] args )
    {
        final String DBURL = "jdbc:postgresql://localhost/pssdemo1";
        final String DBDRIVER = "org.postgresql.Driver";
        final String PMFACTORY = "com.solarmetric.kodo.impl.jdbc.JDBCPersistenceManagerFactory";
        final String USERNAME = "anuser";
        final String PASSWORD = "apassword";

        /* ===== Step 0 : Init the ORB ===== */
        org.objectweb.ccm.CORBA.TheORB.initialize(args);

        /* ===== Step 1 : Create the connector and register storage factories ===== */
        org.omg.CosPersistentState.ConnectorRegistry connectorRegistry = null;
        connectorRegistry = org.omg.CosPersistentState.ConnectorRegistryHelper.narrow(
                org.objectweb.ccm.CORBA.TheORB.resolve_initial_reference("PSS") );
        org.objectweb.openccm.services.pss.common.api.Connector connector = null;
        connector = new org.objectweb.openccm.services.pss.jdo.lib.Connector();
        try
        {
            connector.register_storage_home_factory("PSDL:openccm.objectweb.org/pss/demo1/SH_Person:1.0",
                                                    Class.forName("org.objectweb.openccm.pss.demo1.SH_Person") );
            connector.register_storage_object_factory("PSDL:openccm.objectweb.org/pss/demo1/ST_Person:1.0",
                                                      Class.forName("org.objectweb.openccm.pss.demo1.PersonBaseImpl") );
            connector.register_storage_home_factory("PSDL:openccm.objectweb.org/pss/demo1/SH_Address:1.0",
                                                    Class.forName("org.objectweb.openccm.pss.demo1.SH_Address") );
            connector.register_storage_object_factory("PSDL:openccm.objectweb.org/pss/demo1/ST_Address:1.0",
                                                      Class.forName("org.objectweb.openccm.pss.demo1.ST_Address") );
        }catch(ClassNotFoundException e){
            e.printStackTrace();
        }
        connectorRegistry.register_connector(connector);


        /* ===== Step 2 : Retrieve the connector and create a session (catalog) ===== */
        try
        {
            connector = (org.objectweb.openccm.services.pss.common.api.Connector)
                            connectorRegistry.find_connector("org.objectweb.openccm.services.pss.JDO");
        }catch(org.omg.CosPersistentState.NotFound e){
            e.printStackTrace();
        }

        org.omg.CosPersistentState.Session mySession = null;
        try {
            org.omg.CosPersistentState.Parameter[] parameters = null;
            org.omg.CosPersistentState.Parameter param = null;
            org.omg.CORBA.Any a = null;

            // Init parameters
            parameters = new org.omg.CosPersistentState.Parameter[5];
            a = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
            a.insert_string(DBURL);
            param = new org.omg.CosPersistentState.Parameter("DB_URL", a);
            parameters[0] = param;
            a = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
            a.insert_string(DBDRIVER);
            param = new org.omg.CosPersistentState.Parameter("DB_DRIVER", a);
            parameters[1] = param;
            a = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
            a.insert_string(PMFACTORY);
            param = new org.omg.CosPersistentState.Parameter("PM_FACTORY", a);
            parameters[2] = param;
            a = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
            a.insert_string(USERNAME);
            param = new org.omg.CosPersistentState.Parameter("USER_NAME", a);
            parameters[3] = param;
            a = org.objectweb.ccm.CORBA.TheORB.getORB().create_any();
            a.insert_string(PASSWORD);
            param = new org.omg.CosPersistentState.Parameter("PASSWORD", a);
            parameters[4] = param;

            // create the session
            mySession = connector.create_basic_session( org.omg.CosPersistentState.READ_WRITE.value,
                                                        parameters
                                                      );
            ((org.objectweb.openccm.services.pss.common.api.CatalogBase)mySession).set_connector(connector);
        } catch (Exception e) {
            e.printStackTrace();
        }


        /* ===== Step 3 : Create the Corba Form Object and register it in the NS  ===== */

        // Obtain the ORB.
        org.omg.CORBA.ORB orb = org.objectweb.ccm.CORBA.TheORB.getORB();

        // Obtain the Name Service.
        System.out.println("Obtaining the Name Service...");
        org.omg.CORBA.Object obj = null;
        try {
            obj = orb.resolve_initial_references("NameService");
        } catch(org.omg.CORBA.ORBPackage.InvalidName ex) {
            ex.printStackTrace();
        }
        org.omg.CosNaming.NamingContext nc = org.omg.CosNaming.NamingContextHelper.narrow(obj);

        org.omg.CORBA.Object objPoa = null;
        org.omg.PortableServer.POA rootPOA = null;

        try {
            objPoa = orb.resolve_initial_references( "RootPOA" );
        } catch(org.omg.CORBA.ORBPackage.InvalidName ex) {
            ex.printStackTrace();
        }
        rootPOA = org.omg.PortableServer.POAHelper.narrow( objPoa );
        org.omg.PortableServer.POAManager manager = rootPOA.the_POAManager();

        // Create the CORBA object
        FormImpl form = new FormImpl( mySession );
        obj = form._this( orb );

        // Bind the pss::demo1::form in ths NS
        org.omg.CosNaming.NameComponent[] name = new org.omg.CosNaming.NameComponent[1];
        name[0] = new org.omg.CosNaming.NameComponent();
        name[0].id = "My_PSS_demo1_form";
        name[0].kind = "";

        try {
            nc.rebind(name, obj);
        } catch (org.omg.CosNaming.NamingContextPackage.NotFound err) {
            System.out.println("Object not found ! ");
        } catch (org.omg.CosNaming.NamingContextPackage.CannotProceed err) {
            System.out.println("CannotProceed exception !");
        } catch (org.omg.CosNaming.NamingContextPackage.InvalidName err) {
            System.out.println("InvalidName exception !");
        }

        // Wait for requests
        try {
            manager.activate();
        } catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex) {
            ex.printStackTrace();
        }
        orb.run();
    }
}
