/***************************************************************************/
/* COACH: Component Based Open Source Architecture for                     */
/*        Distributed Telecom Applications                                 */
/* See:   http://www.objectweb.org/                                        */
/*                                                                         */
/* Copyright (C) 2003 Lucent Technologies Nederland BV                     */
/*                    Bell Labs Advanced Technologies - EMEA               */
/*                                                                         */
/* Initial developer(s): Harold Batteram                                   */
/*                                                                         */
/* This library is free software; you can redistribute it and/or           */
/* modify it under the terms of the GNU Lesser General Public              */
/* License as published by the Free Software Foundation; either            */
/* version 2.1 of the License, or (at your option) any later version.      */
/*                                                                         */
/* This library is distributed in the hope that it will be useful,         */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        */
/* Lesser General Public License for more details.                         */
/*                                                                         */
/* You should have received a copy of the GNU Lesser General Public        */
/* License along with this library; if not, write to the Free Software     */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA */
/***************************************************************************/
package org.coach.idltree;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;

public class IdlEditor {       
    public static void display(String title, IdlNode root) {       
        try {
            IdlModel model = new IdlModel(root);
            IdlTreePanel panel = new IdlTreePanel(model);
            final JFrame frame = new JFrame();
            frame.setTitle(title);
            frame.getContentPane().add(panel);
            frame.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
            frame.addWindowListener(new WindowAdapter() {
                public void windowClosed(WindowEvent e) {
                    frame.dispose();
                }
            });
            frame.pack();         
            frame.setVisible(true);            
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * return true if the value has been modified
     */

/*    public static boolean edit(String title, IdlNode root) {       
        try {
            AnyModel model = new AnyModel(root);
            AnyTreePanel panel = new AnyTreePanel(model);
            if (JOptionPane.showConfirmDialog(null, panel, title, JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE) == JOptionPane.CANCEL_OPTION) {           
                return false;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }        
*/
    public static boolean edit(String title, IdlNode root) {       
        try {
            Lock lock = new Lock();
            final JFrame frame = new JFrame();
            frame.setTitle(title);
            final EditPanel panel = new EditPanel(lock, root);
            frame.getContentPane().add(panel);
            frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
            frame.addWindowListener(new WindowAdapter() {
                public void windowClosed(WindowEvent e) {
                    panel.release();
                    frame.dispose();
                }
            });
            frame.pack();         
            frame.setVisible(true);
            
            // wait until the ok button is pressed
            lock.getLock();
            frame.setVisible(false);
            frame.dispose();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }
    
    public static class EditPanel extends JPanel {
        private Lock lock;
        
        public EditPanel(Lock lock, IdlNode root) {
            this.lock = lock;
            setLayout(new BorderLayout());
            IdlModel model = new IdlModel(root);
            IdlTreePanel panel = new IdlTreePanel(model);
            JButton okButton = new JButton("ok");      
            okButton.addActionListener(new java.awt.event.ActionListener() {  
                public void actionPerformed(ActionEvent e) {
                    okButton_actionPerformed(e);
                }
            });
            add(okButton, BorderLayout.SOUTH);
            add(panel, BorderLayout.CENTER);
        }

        void okButton_actionPerformed(ActionEvent e) {
            lock.release();
        }
        
        public void release() {
            lock.release();
        }  
    }
    
    static class Lock {
        private boolean free;
        
        public synchronized void getLock() {
            while(!free) {
                try {
                    wait();
                } catch (Exception e) {
                }
            }
        }
        
        public synchronized void release() {
            free = true;
            notifyAll();
        }
    }
}
