package org.coach.tracing.server.viewer;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.Line2D;

public class IdentityGui extends JComponent  {
    private static ScaleManager sm = ScaleManager.getScaleManager();
    private static float scale;
    private final JPopupMenu popup;
    private JMenuItem hideMenuItem;
    private JMenuItem expandMenuItem;
    private JMenuItem collapseMenuItem;
    private JMenuItem moveRightMenuItem;
    private JMenuItem moveLeftMenuItem;
    
    public final static int min_hsize = 50;
    //the minimum horizontal size of the header component
    public final static int org_hspace = 20;
    //horizontal space between object text
    public final static int org_vspace = 5;
    //vertical space between the text and the rectangle
    public final static int org_rect_width = 20;
    //original width of the rectangle
    public final static int org_rect_height = 5;
    //original height of the rectangle
    
    private Dimension text_dimension = null;
    private String name;
    private Diagram diagram;
    private static int offset;
    private IdentityNode identityNode;

    public IdentityGui(Diagram v, String n, String type, IdentityNode identityNode) {
        super();
        this.diagram = v;
        this.identityNode = identityNode;
        this.name = identityNode.getShortName();
        setFont(sm.getFont());      
        ToolTipManager.sharedInstance().registerComponent(this);
        setToolTipText(n);
        scale = sm.getScale();
        text_dimension = sm.getStringDimension(name + "__", this);
        setSize(Math.max((int)(text_dimension.width + (2 * org_hspace * scale)), (int)(min_hsize * scale)), diagram.getHeight() - 1);

        setLocation(offset, 0);
        offset += getWidth();

        popup = new javax.swing.JPopupMenu();

        hideMenuItem = new javax.swing.JMenuItem();
        hideMenuItem.setText("hide");
        hideMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                hideMenuItemActionPerformed(evt);
            }
        });
        popup.add(hideMenuItem);

        if (!identityNode.isLeaf()) {
            expandMenuItem = new javax.swing.JMenuItem();
            expandMenuItem.setText("expand");
            expandMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    expandMenuItemActionPerformed(evt);
                }
            });
            popup.add(expandMenuItem);
        }

        if (!((IdentityNode)identityNode.getParent()).isRoot()) {
            collapseMenuItem = new javax.swing.JMenuItem();
            collapseMenuItem.setText("collapse");
            collapseMenuItem.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    collapseMenuItemActionPerformed(evt);
                }
            });
            popup.add(collapseMenuItem);
        }

        moveRightMenuItem = new javax.swing.JMenuItem();
        moveRightMenuItem.setText("move right");
        moveRightMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moveRightMenuItemActionPerformed(evt);
            }
        });
        popup.add(moveRightMenuItem);
                
        moveLeftMenuItem = new javax.swing.JMenuItem();
        moveLeftMenuItem.setText("move left");
        moveLeftMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moveLeftMenuItemActionPerformed(evt);
            }
        });
        popup.add(moveLeftMenuItem);

        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent e) {
                if (e.isPopupTrigger()) {
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
            public void mouseReleased(java.awt.event.MouseEvent e) {
                if (e.isPopupTrigger()) {
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
        });
    }
            
    public static void reset() {
        offset = 0;
    }
    
    public int getXoffset() {
        return getX() + getWidth() / 2;
    }
        
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2 = (Graphics2D)g;
        g2.setRenderingHints(sm.getHints());
        g2.setStroke(sm.getStroke());
        g2.setFont(sm.getFont());
        g2.setColor(Color.blue);
        //draw the string
        g2.drawString(name, (int)(org_hspace * scale), (text_dimension.height - 1));
        g2.setColor(Color.black);
        //draw the black rectangle
        g2.fillRect((int)(((getWidth() - 1) - (org_rect_width * scale)) / 2), (int)(text_dimension.height + (org_vspace * scale)), (int)(org_rect_width * scale), (int)(org_rect_height * scale));
        // draw the line
        g2.draw(new Line2D.Float(getWidth() / 2, text_dimension.height + (org_vspace * scale) + 1, getWidth() / 2, getHeight() - 1));
    }

    private void expandMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        identityNode.expand();
        diagram.repaintEvents();
    }

    private void collapseMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        identityNode.collapse();
        diagram.repaintEvents();
    }
        
    private void hideMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        identityNode.setHidden(true);
        diagram.repaintEvents();
    }

    private void moveRightMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        IdentityNode.moveRight(identityNode);
        diagram.repaintEvents();
    }

    private void moveLeftMenuItemActionPerformed(java.awt.event.ActionEvent evt) {
        IdentityNode.moveLeft(identityNode);
        diagram.repaintEvents();
    }
}

