/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/

package org.objectweb.ccm.descriptor.componentassembly.ccm;

//Importation of the needed packages
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;
import org.objectweb.ccm.descriptor.softpkg.*;
import org.objectweb.ccm.descriptor.softpkg.beans.*;

/**
 * Deployer class in charge to resolve the specified destination. 
 * It supports to destination kind: CS and Node. <br>
 * <ul>
 *  <li>CS destination refer to the older association betweeen  
 *  a component server and a component installation. This server is 
 *  started bu JCS_start script.The destination precised is in fact the name 
 *  CS started on target host.
 *  </li>
 * 
 * <li>Node destination refer to the ServerActivator/ComponentInstallation
 * association defined in the CCM specification. The destination precised 
 * is in fact the name of the started node on target host.
 * <li>
 * </ul>
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */

public class DestinationDeployer {
    
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /** A server which hold a ComponentServer and a ComponentInstantiation*/
    private org.objectweb.ccm.Deployment.Server server ;
    /** The target server activator */
    private org.omg.Components.Deployment.ServerActivator sa;
    /** The target component installation*/
    private org.omg.Components.Deployment.ComponentInstallation ci;
    /** The node manager target*/
    private org.ist.coach.DCI.NodeManager node;

    /** Namecomponent use to found object*/
    private org.omg.CosNaming.NameComponent [] ncomp;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    public DestinationDeployer(){
        server=null;
        sa=null;
        ci=null;       
    }


    // ==================================================================
    //
    // Internals methods.
    //
    // ==================================================================

    /**
     * Private method allowing to retrieve server reference with the name 
     * service.
     *
     * @param destination the target server name to retrive
     * @return the server found
     */
    private 
    org.objectweb.ccm.Deployment.Server 
    getServer(java.util.Hashtable deployTable,
              String destination)
    throws Exception
    {
        if(server==null){
            String dest=destination.substring(destination.indexOf("CS:")+3,  
                                              destination.length());
            dest=dest.trim();
            //retrive server composition
            server=org.objectweb.ccm.Deployment.ServerHelper.narrow(getRef(deployTable,dest)); 
        }
        return server;
    }
    
    /**
     *Internal method to retrieve ServerActivator.
     *
     * @param destination the target server activator name to retrive
     * @return the server activator found
     */
    private 
    org.omg.Components.Deployment.ServerActivator
    getServerActivator(java.util.Hashtable deployTable,
                       String destination)
    throws Exception
    {
        if(sa==null){
            String dest=System.getProperty("Node_ns_dir")+
                        destination.substring(destination.indexOf("NODE:")+5,  
                                              destination.length());
            dest=dest.trim();
            //Retrieve the Server Activator Reference
            /*sa=org.omg.Components.Deployment.ServerActivatorHelper.
              narrow(getRef(deployTable,"ServerActivator"+dest)); */
            sa=
                (org.ist.coach.DCI.NodeManagerHelper.
                 narrow(getRef(deployTable,dest))).provide_server_activator();
        }
        return sa;                               
    }
    
    /**
     *Internal method to retrieve ComponentInstallation
     *
     * @param destination the target component installation  name to retrive
     * @return the component installation found     
     */
    private org.omg.Components.Deployment.ComponentInstallation
    getComponentInstallation(java.util.Hashtable deployTable,
                             String destination)
    throws Exception
    {
        if(ci==null){
            String dest=System.getProperty("Node_ns_dir")+
                        destination.substring(destination.indexOf("NODE:")+5,  
                                              destination.length());
            dest=dest.trim();
            //Retrieve the Server Activator Reference
            /*  ci=org.omg.Components.Deployment.ComponentInstallationHelper.
                narrow(getRef(deployTable,"ComponentInstallation"+dest)); */
            ci=
                (org.ist.coach.DCI.NodeManagerHelper.
                 narrow(getRef(deployTable,dest))).provide_component_installation();
        }
        return ci;                               
    }
    
    /**
     * Provide a corba reference according to the name give in 
     * parameter.
     *
     * @param ref the name of corba reference to retrieve
     * @return the ref corba found
     **/
    private  org.omg.CORBA.Object 
    getRef(java.util.Hashtable deployTable,String ref)
    throws Exception
    {
        org.omg.CosNaming.NamingContextExt  rootContext=
            org.omg.CosNaming
            .NamingContextExtHelper
            .narrow((org.omg.CORBA.Object)deployTable.get("__name_service__"));    
        
        ncomp= rootContext.to_name(ref);
          
        return rootContext.resolve(ncomp);       
    }



        /**
     * Method providing the node matching with the node name
     * give in parameter.
     *
     * @param deployTable the internal table use to store information.
     * @param dest the node name
     * @return the target NodeManager
     */
    private  org.ist.coach.DCI.NodeManager 
    getNode(java.util.Hashtable deployTable,
            String dest)
    {
        if(node==null)
            {
                try{
                    org.ist.coach.DCI.DCIInformation info=
                        (org.ist.coach.DCI.DCIInformation)
                        deployTable.get("__dci_information__");
                    
                    node = info.get_node_manager(dest.trim());
                    
                    if(deployTable.get("__assembly_manager__")!=null)
                        ((org.objectweb.openccm.OpenCCM_DCI.AssemblyManager)
                         deployTable.get("__assembly_manager__")).connect_node_managers(node);
                }
                catch(Exception e){
                    e.printStackTrace();
                    System.err.println("Cannot retrieves node "+dest+" with DCI");
                }
            }
        return node;            
    }


    /**
     * Method providing the component installation matching with the
     * component installation name give in parameter.
     *
     * @param deployTable the internal table use to store information.
     * @param destination the node name
     * @return the target NodeManager
     */
    private org.omg.Components.Deployment.ComponentInstallation
    getComponentInstallationDCI(java.util.Hashtable deployTable,
                                String destination)
    {
        String dest=destination.substring(destination.indexOf("DCI:")+4,  
                                          destination.length());
        try{
            return getNode(deployTable,dest).provide_component_installation();
        }
        catch(Exception e){
            System.err.println("Destination invalide:"+dest);
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Method providing the server activator matching with the
     * server activator name give in parameter.
     *
     * @param deployTable the internal table use to store information.
     * @param destination the node name
     * @return the target NodeManager
     */    
    private org.omg.Components.Deployment.ServerActivator
    getServerActivatorDCI(java.util.Hashtable deployTable,
                          String destination)
    {
        String dest=destination.substring(destination.indexOf("DCI:")+4,  
                                          destination.length());
        try{
            return getNode(deployTable,dest).provide_server_activator();
        }
        catch(Exception e){
            System.err.println("Destination invalide:"+dest);
            e.printStackTrace();
        }
       
        return null;

    }
    


    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * TODO : Process exception mecanism
     * Definition of method allowing to retrieve a component server
     * reference pointed by the destination.<br>
     *  <destination>CS:"ServerName"</destination> <br>
     *  <destination>Node:"NodeName"</destination> <br>
     *
     * @param deployTable the internal table use to store information.
     * @param target the target destination
     * @return the target component server
     **/
    public  org.omg.Components.Deployment.ComponentServer
    getComponentServer(java.util.Hashtable deployTable,
                       Destination target)
    {
        try{
            String destination=target.getValue();
            
            if(destination.trim().startsWith("CS:"))
                return getServer(deployTable,destination).provide_component_server();
            
            //initalize the component server configuration                          
            org.omg.Components.ConfigValue[] config
                = new org.omg.Components.ConfigValue[0];
            
            if (destination.trim().startsWith("NODE:")){
                
                // create and return the component server reference                      
                return getServerActivator(deployTable,destination)
                    .create_component_server(config);
            }
            if(destination.trim().startsWith("DCI:")){
                
                org.omg.Components.Deployment.ComponentServer cs=
                    getServerActivatorDCI(deployTable,destination)
                    .create_component_server(config);
                // Connect the component server to the assembly manager
                if(deployTable.get("__assembly_manager__")!=null)
                    ((org.objectweb.openccm.OpenCCM_DCI.AssemblyManager)
                     deployTable.get("__assembly_manager__")).connect_component_servers(cs);

                return cs;
            }
        }
        catch(Exception e){e.printStackTrace();}        
        return null;
    }
    
    /**
     * Definition method provide the component server pointed
     * by the destination.<br>
     *  <destination>CS:"ServerName"</destination> <br>
     *  <destination>Node:"NodeName"</destination> <br>
     *
     * @param deployTable the internal table use to store information.
     * @param target the target destination
     * @return the target component installation
     */
    public  org.omg.Components.Deployment.ComponentInstallation
    getComponentInstallation(java.util.Hashtable deployTable,
                             Destination target)
    {
        String destination=target.getValue();
        try{   
            //Destination refer server composition:CS+CI
            if(destination.trim().startsWith("CS:"))
                //return Component Installation segmnet  
                return getServer(deployTable,destination).provide_install();
        
            //Destination refer to a node 
            if (destination.trim().startsWith("NODE:"))
                return getComponentInstallation(deployTable,destination);                    
            if (destination.trim().startsWith("DCI:"))
                return getComponentInstallationDCI(deployTable,destination);
        }
        catch(Exception e)
            {
                System.out.println("Cannot retrieve the ComponentInstallation:"+
                                   e.getMessage());
                e.printStackTrace();
            }
        return null;         
    }
    

    /**
     * Remove a component server.
     *
     * @param cs the componentserver to remove
     */
    public void 
    removeComponentServer(org.omg.Components.Deployment.ComponentServer cs)
    throws Exception
    {
        if(sa!=null)
            sa.remove_component_server(cs);       
        else
            cs.remove();
    }
    
}
