/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/


package org.objectweb.ccm.descriptor.softpkg.ccm;

// Import all the packages required
import org.objectweb.ccm.descriptor.softpkg.*;
import org.objectweb.ccm.descriptor.softpkg.beans.*;

/**
 *
 * Fileinarchive is used to read xml file information in the archive.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */

public class FileinarchiveDeployer {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    /*Server socket used to upload the archive*/
    private java.net.ServerSocket serverSocket;

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Retrieve a reader on the file denote by this fileinarchive 
     * element
     * @param deployTable the internal table use to store information
     * @param fa the archive file to used
     */
    public java.io.InputStreamReader 
    getInputStreamReader(java.util.Hashtable deployTable, 
                         Fileinarchive fa)
    throws java.io.IOException
    {    
        return  new java.io.InputStreamReader
            (getInputStream(deployTable,fa));
    }
    
    /**
     * Retrieve a inputstream on the file denote by this fileinarchive 
     * element
     * @param deployTable the internal table use to store information
     * @param fa the archive file to used
     */
    public java.io.InputStream
    getInputStream(java.util.Hashtable deployTable,
                   Fileinarchive fa)
    throws  java.io.IOException
    {
        java.util.zip.ZipFile zf=(java.util.zip.ZipFile)deployTable.get("__car_zip__");
        return zf.getInputStream
            (zf.getEntry(fa.getName().trim()));
    }
    /**
     * Start a specific Http server in charge to upload the specific 
     * file in the archive.
     * The server will process only one request. This server is started
     * in another thread.
     *
     * @param deployTable the internal table use to store information
     * @param fa the archive file to used
     * @return The URL to use for downloading the file.
     */
    public String
    startHttpServer(java.util.Hashtable deployTable,
                    Fileinarchive fa) 
    throws java.io.IOException
    {
        serverSocket=(java.net.ServerSocket)
            deployTable.get("__server_socket__");
        
        (new Thread
            (new org.objectweb.ccm.deploytool.MicroServerHttp
                (serverSocket,
                 getInputStream(deployTable,fa),
                 fa.getName().trim()))).start();
        return 
            "http://" + 
            java.net.InetAddress.getLocalHost().getHostAddress() + ':' +
            serverSocket.getLocalPort() + '/' +
            fa.getName().trim();
    }


}
