/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Christophe Demarey.
Contributor(s): _______________________________.

====================================================================*/

package org.objectweb.ccm.runtime.cif.lib;

/**
 * Abstract base class for all session component segment classes
 * generated from CIDL.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public abstract class SessionComponentSegmentBase
              extends ComponentSegmentBase
           implements org.omg.Components.SessionComponent
{
    // ==================================================================
    //
    // Internal State.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    SessionComponentSegmentBase()
    {
        // Do nothing.
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for OMG IDL ::Components::EnterpriseComponent
    //
    // ==================================================================

    // Nothing as the interface is empty.

    // ==================================================================
    //
    // Public methods for OMG IDL ::Components::ExecutorLocator
    //
    // ==================================================================

    // Nothing as the interface is implemented in ComponentSegmentBase.

    // ==================================================================
    //
    // Public methods for OMG IDL ::Components::SessionComponent
    //
    // ==================================================================

    /**
     * Sets the session component context.
     *
     * Must be implemented in subclasses.
     *
     * @param context The session component context.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    abstract public void
    set_session_context(org.omg.Components.SessionContext context)
    throws org.omg.Components.CCMException;

    /**
     * Container callback to signal that the component is activated.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_activate()
    throws org.omg.Components.CCMException
    {
        getLogger().trace(this,
            "Activating session component: Do nothing by default");

        // Do nothing by default!
    }

    /**
     * Container callback to signal that the component is activated.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_passivate()
    throws org.omg.Components.CCMException
    {
        getLogger().trace(this,
            "Passivating session component: Do nothing by default");

        // Do nothing by default!
    }

    /**
     * Container callback to signal that the component is removed.
     *
     * @throw org.omg.Components.CCMException For any problems.
     */
    public void
    ccm_remove() 
    throws org.omg.Components.CCMException
    {
        getLogger().trace(this,
            "Removing session component: Do nothing by default");

        // Do nothing to do by default!
    }
}
