/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.api;

import org.objectweb.corba.generator.translator.idl2java.api.PSDL_JavaTranslator;
import org.objectweb.corba.generator.java.ast.api.InterfaceObject;
import org.objectweb.corba.generator.java.ast.api.ClassObject;
import org.objectweb.corba.generator.java.ast.api.MethodObject;


/**
 * This is the interface for State Member Mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public interface StateMemberMapping
{
    /**
     * Map a Storage Type State Member.
     *
     * @param translator - A Java utility class to translate types.
     * @param obj - The StorageType mapping object.
     * @param finalAccessor - True if accessors must be final.
     */
    public void
    toJavaAccessors(PSDL_JavaTranslator translator,
                    InterfaceObject obj,
                    boolean finalAccessor);

    /**
     * Map a Read Only Storage Type State Member to a user non-available modifier.
     *
     * @param translator - A Java utility class to translate types.
     * @param clazz - The StorageType mapping object.
     */
    public void
    toJavaReadOnlyStateMemberModifier(
            PSDL_JavaTranslator translator,
            ClassObject clazz);

    /**
     * Map a Storage Type State Member to an attribute.
     *
     * @param translator - A Java utility class to translate types.
     * @param clazz - The StorageType mapping object.
     * @param st - The StorageType declaration.
     */
    public void
    toJavaAttribute(PSDL_JavaTranslator translator,
                    ClassObject clazz,
                    org.objectweb.corba.ast.api.StorageTypeDecl st);

    /**
     * Map a state member to a java parmeter.
     *
     * @param translator - A Java utility class to translate types.
     * @param method - The method that will contain parameters.
     **/
    public void
    toJavaParameter(PSDL_JavaTranslator translator,
                    MethodObject method);
}
