/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.lib;

/** To access AST. */
import org.objectweb.corba.ast.api.AbstractStorageTypeDecl;
import org.objectweb.corba.ast.api.StorageTypeDecl;
import org.objectweb.corba.ast.api.AbstractStorageHomeDecl;
import org.objectweb.corba.ast.api.StorageHomeDecl;
import org.objectweb.corba.ast.api.DeclarationKind;
import org.objectweb.corba.ast.api.Scope;

/** To access Java AST. */
import org.objectweb.corba.generator.java.lib.*;
import org.objectweb.corba.generator.java.ast.api.*;
import org.objectweb.corba.generator.java.ast.lib.*;

/** Others. */
import java.util.Vector;
import org.objectweb.corba.generator.common.lib.GenerationException;


/**
 * This class generates PSDL to Java Mappings.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class PSDL2JavaGenerator
     extends org.objectweb.corba.generator.common.lib.GeneratorBase
  implements org.objectweb.corba.generator.psdl.api.PSDL2JavaGenerator
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The java repository */
    private Repository repository_;

    /** Utility class to convert types in Java*/
    public org.objectweb.corba.generator.translator.idl2java.api.PSDL_JavaTranslator translator_;

    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     */

    public PSDL2JavaGenerator(org.objectweb.corba.ast.api.AST ast)
    {
        super(ast);
        // Init internal state
        repository_ = null;
        translator_ = new org.objectweb.corba.generator.translator.idl2java.lib.PSDL_JavaTranslator();
    }

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Useful methods.
    //
    // ==================================================================


    // ===========================================================
    //
    // Others methods.
    //
    // ===========================================================

    /**
     * Get the generator translator utility class.
     *
     * @return The translator.
     **/
    public org.objectweb.corba.generator.translator.idl2java.api.PSDL_JavaTranslator
    getTranslator()
    {
        return translator_;
    }

    /**
     * Get the java Repository.
     *
     * @return The translator.
     **/
    public org.objectweb.corba.generator.java.ast.api.Repository
    getJavaRepository()
    {
        return repository_;
    }


    /**
     * Generate java files for the PSDL JAVA Mappings.
     *
     * @param base_dir - The base directory where files will be generated.
     * @param scope - The scope to map.
     **/
    public void
    psdl_to_java(String base_dir, Scope scope)
    throws GenerationException
    {
        java.io.File out_dir = null;
        repository_ = new RepositoryImpl();
        Vector vect = null;
        AbstractStorageTypeDecl[] ast_array = null;
        StorageTypeDecl[] st_array = null;
        AbstractStorageHomeDecl[] ash_array = null;
        StorageHomeDecl[] sh_array = null;

        // Check and create the output directory
        try{
            out_dir = org.objectweb.corba.generator.common.lib.FileManager.mkdir(base_dir);
        }catch(GenerationException ex){
            System.err.println(ex.getMessage());
            return;
        }

        /** Get all Abstract Storage Types contained in the file scope and map **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_abstract_storage_type);
        ast_array = (AbstractStorageTypeDecl[])vect.toArray(new AbstractStorageTypeDecl[0]);
        for (int i=0; i<ast_array.length; i++)
        {
            org.objectweb.corba.generator.psdl.api.JavaMappingObject ast
                = new AbstractStorageTypeMapping(ast_array[i]);
            ast.toJava(this);
        }

        /** Get all Abstract Storage Homes contained in the file scope and map **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_abstract_storage_home);
        ash_array = (AbstractStorageHomeDecl[])vect.toArray(new AbstractStorageHomeDecl[0]);
        for (int i=0; i<ash_array.length; i++)
        {
            org.objectweb.corba.generator.psdl.api.JavaMappingObject ash
                = new AbstractStorageHomeMapping(ash_array[i]);
            ash.toJava(this);
        }

        /** Get all Storage Types contained in the file scope and map **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_storage_type);
        st_array = (StorageTypeDecl[])vect.toArray(new StorageTypeDecl[0]);
        for (int i=0; i<st_array.length; i++)
        {
            org.objectweb.corba.generator.psdl.api.StorageTypeMapping st
                = new StorageTypeMapping(st_array[i]);
            st.toJava(true, this);
        }
        JdoXmlGenerator jdo_xml_gen = new JdoXmlGenerator(this);
        jdo_xml_gen.initialize("psdl_java");
        jdo_xml_gen.to_jdo_xml(base_dir, vect);

        /** Get all Storage Homes contained in the file scope and map **/
        vect = getAllDeclarations(scope, DeclarationKind.dk_storage_home);
        sh_array = (StorageHomeDecl[])vect.toArray(new StorageHomeDecl[0]);
        for (int i=0; i<sh_array.length; i++)
        {
            org.objectweb.corba.generator.psdl.api.JavaMappingObject sh
                = new StorageHomeMapping(sh_array[i]);
            sh.toJava(this);
        }

        // Generate Java Files
        org.objectweb.corba.generator.java.lib.JavaGenerator java_gen = new JavaGenerator();
        java.util.Vector v = new java.util.Vector();
        v.add("psdl2java.vm");
        java_gen.initialize("PSDL2JAVA", v);
        java_gen.generate(repository_, base_dir); 
    }
}
