/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.corba.generator.psdl.lib;

/** To access AST. */
import org.objectweb.corba.ast.api.TypeKind;
import org.objectweb.corba.ast.api.DeclarationKind;
import org.objectweb.corba.ast.api.Declaration;
import org.objectweb.corba.ast.api.StorageTypeStateMemberDecl;
import org.objectweb.corba.ast.api.StorageTypeStoreDirectiveDecl;

/**
 * This class implements useful methods dor PSDL Mapping.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class PSDLMappingTools
     extends PSDLTools
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     */

    public PSDLMappingTools()
    {
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Get the mapping type to marshal a typeref.
     *
     * @param typeref - A typeref to marshall.
     *
     * @return The marshalled type.
     */
    public static String
    getMarshalledType(org.objectweb.corba.ast.api.TypeRef typeref)
    {
        TypeKind tk = typeref.getTypeKind();

        if (tk == TypeKind.tk_alias)
        {
            // Get the "native" type
            org.objectweb.corba.ast.api.AliasDecl alias;

            alias  = (org.objectweb.corba.ast.api.AliasDecl)typeref;
            return getMarshalledType( alias.getType() );
        }

        if ( (tk == TypeKind.tk_sequence)
             || (tk == TypeKind.tk_array) )
        {
            org.objectweb.corba.ast.api.TypeRefWithContent content = null;
            java.lang.String str = null;

            content = (org.objectweb.corba.ast.api.TypeRefWithContent)typeref;
            str = getMarshalledType( content.getContentType() );
            return str + "[]";
        }

        if ( (tk == TypeKind.tk_struct)
             || ( tk == TypeKind.tk_union)
             || ( tk == TypeKind.tk_value_box)
             || ( tk == TypeKind.tk_any)
             || ( tk == TypeKind.tk_value) )
        {
            return "byte[]"; // A BLOB
        }
        else if ( tk == TypeKind.tk_interface)
        {
            return "java.lang.String"; // The IOR
        }

        return null;
    }

    /**
     * Get the mapping type to marshal a psdl state member.
     *
     * @param st - The StorageType declaration.
     * @param state - The state member to marshall.
     *
     * @return The marshalled type.
     */
    public static String
    getMarshalledType(org.objectweb.corba.ast.api.StorageTypeDecl st,
                      StorageTypeStateMemberDecl state)
    {
        org.objectweb.corba.ast.api.PsdlTypeRef psdltyperef = null;
        org.objectweb.corba.ast.api.TypeRef idl_type = null;

        psdltyperef = state.getType();
        idl_type = psdltyperef.getTypeRef();

        if (idl_type != null)
        {
            return getMarshalledType(idl_type);
        }
        else
        {
            // type is an abstract storagetype
            if (!psdltyperef.isRef())
            {
                org.objectweb.corba.generator.translator.idl2java.api.PSDL_JavaTranslator translator = null;
                Declaration[] decls = null;
                String sd_name = null;

                // Find a store directive for this state
                decls = st.getContents(true, DeclarationKind.dk_storage_type_store_directive);
                sd_name = state.getName() + "_store_directive";
                for (int i=0;i<decls.length;i++)
                {
                    if (sd_name.compareTo(decls[i].getName()) == 0)
                    {
                        StorageTypeStoreDirectiveDecl sd = (StorageTypeStoreDirectiveDecl)decls[i];
                        translator = new org.objectweb.corba.generator.translator.idl2java.lib.PSDL_JavaTranslator();
                        return translator.toJava(sd.getType());
                    }
                }
            }
            return null;
        }
    }
}
