/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.translator.ast2idl.lib;

// To acces AST CategoryKind
import org.objectweb.corba.ast.api.CategoryKind;

/**
 * Utility class to translate CIDL types in string format.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class AST_CIDLTranslator
     extends org.objectweb.corba.generator.translator.ast2idl.lib.AST_PSDLTranslator
  implements org.objectweb.corba.generator.translator.ast2idl.api.AST_CIDLTranslator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     *  The default constructor.
     **/
    public
    AST_CIDLTranslator()
    {
        super();
        keywords_.add("bindsTo");
        keywords_.add("composition");
        keywords_.add("delegatesTo");
        keywords_.add("entity");
        keywords_.add("executor");
        keywords_.add("facet");
        // Already defined in PSDL
        // keywords_.add("implements");
        keywords_.add("process");
        keywords_.add("proxy");
        keywords_.add("segment");
        keywords_.add("service");
        keywords_.add("session");
        // Already defined in PSDL
        // keywords_.add("storagehome");
        keywords_.add("storedOn");
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Translate a cidl category kind into its string format.
     *
     * @param ck - The category kind to translate.
     *
     * @return The resulting string.
     **/
    public String
    toCIDL(CategoryKind ck)
    {
        if (ck == CategoryKind.ck_entity)
            return "entity";
        else if (ck == CategoryKind.ck_process)
            return "process";
        else if (ck == CategoryKind.ck_service)
            return "service";
        else
            return "session";
    }

}
