/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/



package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;
import org.objectweb.ccm.CORBA.*;

// Package dependencies
import org.objectweb.util.cmdline.api.CommandLine;
import org.objectweb.util.cmdline.lib.ApplicationBase;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.ccm.descriptor.componentassembly.ccm.*;
import org.objectweb.ccm.descriptor.componentassembly.beans.*;
import org.objectweb.ccm.descriptor.componentassembly.*;

/**
 * AssemblyImpl is the top level implementation of the 
 * assembly behaviour. In the DCI implementation the assembly interface
 * is provided by a AssemblyManager component. So many connection of 
 * this component are used for the initialization of the deployment 
 * process.
 *
 * @author <a href="mailto:frederic.briclet@lifl.fr">Briclet Frederic</a>
 *
 * @version 0.1
 */
public class AssemblySegImpl 
    extends  org.objectweb.openccm.OpenCCM_DCI.
             AssemblyManagerSessionComposition.AssemblySeg
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    // xml handler path if 
    private String xmlFilePath;
    // xml handler path 
    private String xml;
    
    // Console for emittion of message
    private org.objectweb.util.cmdline.api.Console console;
    
    //Root  of xml tree cad representation
    private ComponentassemblyBean assemblyObject;
    
    //Table of component handle order by id key
    private java.util.Hashtable general;
    
    // State of the assembly
    private org.omg.Components.Deployment.AssemblyOperations etat;
    
    // The assembly manager component
    private AssemblyManager am;   
    //Reference on the NameService
    private org.omg.CosNaming.NamingContextExt ns ;
    //Name component involve in the NS registration
    private org.omg.CosNaming.NameComponent [] ncomp;


    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    /**
     * Internal function to register the assembly with the specified name 
     * in cad descriptor in order to retrieve and tear_down the assembly later.
     */
    private void 
    registerAssembly()
    {
        try{
            // retrieve the corba reference from the assembly object
            org.omg.Components.Deployment.Assembly assemblyRef=
                ((org.objectweb.openccm
                  .OpenCCM_DCI.AssemblyManagerCCM)get_context())
                .provide_assembly(); 
            
            // Obtain the Name Service.
            ns=org.omg.CosNaming.NamingContextExtHelper
                .narrow(org.objectweb.ccm.CORBA
                        .TheNameService
                        .getNamingContext()
                        .getNamingContext());
            
            // Bind the server object in the name service.
            ncomp= org.objectweb.ccm.descriptor
                    .componentassembly
                    .ccm.RegisterwithnamingDeployer
                        .rebind(ns,
                                System.getProperty("Assemblies_ns_dir")+
                                assemblyObject.getId(),
                                assemblyRef);
            }
        catch(Exception e){
             System.err.println("Following exception occur during assembly registration");
             System.err.println(e.getMessage());
             System.err.println("Cannot register the assembly"+
                                System.getProperty("Assemblies_ns_dir") +
                                assemblyObject.getId());
             return ;
        }
        System.out.println("The Assembly"+
                               System.getProperty("Assemblies_ns_dir")+
                               assemblyObject.getId()+" is referenced");
    }
    /**
     * Unregister the assembly from the NameService
     */
    private void
    unregisterAssembly()
    {
        try{
            ns.unbind(ncomp);
        }
        catch(Exception e){
             System.err.println("Following exception occur during assembly unregistration");
             System.err.println(e.getMessage());
             System.err.println("Cannot unregister the assembly"+
                                System.getProperty("Assemblies_ns_dir")+
                                assemblyObject.getId());
        }
    }

    // ==================================================================
    //
    // Internal class.
    //
    // ==================================================================
     /**
     * Inactive state for the assembly
     */
    private class AssemblyImplInactive 
        implements  org.omg.Components.Deployment.AssemblyOperations 
    { 
        
        private java.io.File tmpZipFile;
             
        public void setupDeployTable()
        {
            try{
                general.put("__home_finder__",
                            get_context().get_connection_home_finder());
                general.put("__home_registration__",
                            get_context().get_connection_home_registration());
                general.put("__name_service__",
                            get_context().get_connection_naming_service());
                general.put("__dci_information__",
                            get_context().get_connection_dci_information());
                general.put("__assembly_manager__",am);
                general.put("__component_list__",new java.util.LinkedList());
                //general.put("__trader__",get_context().get_connection_trader());
                /*Put a reference to the server socket to use*/
                java.net.ServerSocket s = new java.net.ServerSocket(0); 
                general.put("__server_socket__",s);

            }
            catch(Exception e){
                System.err.println("Problem occured during setting of the deployTable");
            }
        }


        public void 
        build() 
        throws org.omg.Components.CreateFailure 
        {
            try{
                
                //console.message("Load the tree");
                loadCADTree();
                
                setupDeployTable();

                /*SAve usefull information to deployment*/
                general.put("BaseDir",org.objectweb.ccm.deploytool.DeploymentProcess.getBaseDir());	
                //general.put("__Console__",console);
                /*Start the deployment operation*/                
		
                ((ComponentassemblyDeployer)
                 assemblyObject
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).deployPartial(general,assemblyObject);
                
                ((ComponentassemblyDeployer)
                 assemblyObject
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).deployFinal(general,assemblyObject);
                
                /*Setup the component to configuration complete state*/
                for (java.util.ListIterator it =
                         ((java.util.LinkedList)
                          general.get("__component_list__")).listIterator(0);
                     it.hasNext() ; )
                    {
                        try {
                            ((org.omg.Components.CCMObject)
                             general.get((String)it.next()))
                                .configuration_complete();
                        }
                        catch (java.lang.ClassCastException exception) {
                            /*Nothing to do for non component objet like console*/
                        }
                        catch (org.omg.Components.InvalidConfiguration exception) 
                            {
                                exception.printStackTrace();
                                //console.error("Could not switch a CCM Object to configuration complete");
                                return ;
                            }
                    }
                tmpZipFile.delete();
            }
            catch(Exception e)
                {
                    e.printStackTrace(); throw new org.omg.Components.CreateFailure();
                }
            etat=(org.omg.Components.Deployment.AssemblyOperations) new AssemblyImplActive();		
            registerAssembly() ;
            
        }
	
	    	
        /**
         * Private function allowing inactive assembly to load 
         * tree representation of the xml cad file
         */
        private void 
        loadCADTree()
        throws org.omg.Components.CreateFailure
        {
            
            try {
                /*Creating the java file like with .CAD file*/
                if(xml!=null)
                    {
                        /*Creating the java file like with .CAD file*/
                        java.io.StringReader reader= 
                            new java.io.StringReader (xml);
                        
                        assemblyObject
                            = org.objectweb.ccm.descriptor
                            .componentassembly.beans.ComponentassemblyBeanImpl
                            .unmarshalBean(reader,true);
                    }
                else
                    {   
                        
                        //java.io.File assemblyFile= 
                        //  new java.io.File (xmlFilePath);
                        java.net.URL url= new java.net.URL(xmlFilePath);
                        //System.err.println("Non du fichier:"+url.getFile());
                        tmpZipFile =java.io.File.createTempFile(System.currentTimeMillis()+"","aar");
                        //tmpZipFile = new java.io.File("/home/briclet/"+url.getFile());
                        //System.err.println("Nv fichier cre"+tmpZipFile.createNewFile());

                        java.io.FileOutputStream fos= new java.io.FileOutputStream(tmpZipFile);

                        
                        java.io.InputStream bis=url.openStream();
                       
                        byte [] byteArray= new byte[32000];
                        int readed=readed=bis.read(byteArray,0,32000);                        
                        
                        System.err.print("Downloading "+xmlFilePath+" [");
                        while(readed>0)                      
                            {    //cumul+=readed;
                                fos.write(byteArray,0,readed);
                                readed=bis.read(byteArray,0,32000);
                                System.err.print("*");
                            }
                        System.err.println("]");
                        //  byte[] data= new byte[bis.available()];
//                          bis.read(data);
//                          fos.write(data);
                        fos.flush();
                        fos.close();
                        
                        java.util.zip.ZipFile zipFile= 
                            new java.util.zip.ZipFile(tmpZipFile);

                        general.put("__car_zip__",zipFile);

                        java.io.InputStreamReader reader ;

                        for(java.util.Enumeration enum=zipFile.entries();enum.hasMoreElements();)
                            {

                                java.util.zip.ZipEntry ze=(java.util.zip.ZipEntry)enum.nextElement();
                                //System.out.println("Filename:"+ze.getName());
                                if(ze.getName().endsWith(".cad"))
                                    {
                                  
                                        reader=new java.io.InputStreamReader
                                            (zipFile.getInputStream(ze));
                                             
                                        
                                        assemblyObject
                                            = org.objectweb.ccm.descriptor
                                            .componentassembly.beans.ComponentassemblyBeanImpl
                                            .unmarshalBean(reader,true);//(assemblyFile);
                                    }

                            }
                            /* java.io.InputStreamReader reader=
                               new java.io.InputStreamReader
                               ((new java.net.URL(xmlFilePath)).openStream());*/
                                    

                       
                    }                
            } 
            catch (Exception ex) 
                {
                    ex.printStackTrace();
                    console.error("Could not unmarshal the assembly file");
                    throw new org.omg.Components.CreateFailure();
                }
            //console.message("Got the assembly descriptor");
        }
        
        /**
         * Break connection between composant and destroy them
         */	
        public void tear_down() throws org.omg.Components.RemoveFailure
        {
            throw new  org.omg.Components.RemoveFailure();	
        }
        
        /**
         * Return the current assembly state	
         */
        public org.omg.Components.Deployment.AssemblyState get_state()
        {
            return 	org.omg.Components.Deployment.AssemblyState.INACTIVE;
            
        }
    }	
    
    /**
     * Internal class which represent an deploied assembly, only 
     * tear_down() and get_state() operation are available 
     */
    private class AssemblyImplActive 
        implements org.omg.Components.Deployment.AssemblyOperations
    {
        /**
         * Alway thrown CreateFailure exception because the assembly
         * is already deploied. This function is non available
         */
        public void build() throws org.omg.Components.CreateFailure
        {
            throw new org.omg.Components.CreateFailure();
        }
        
        /**
         *	Tear_down the assembly in 2 part: 
         *		First tear_down connections
         *		Secondly erease instantiation
         *  Switch the internal state of the assembly to inactive state.
         */
        public void tear_down() throws org.omg.Components.RemoveFailure 
        {
            try{
                //Unregister the assembly from NS
                 unregisterAssembly();
                 
                ((ComponentassemblyDeployer)
                 assemblyObject
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance())
                    .tearDownConnections(general,
                                         (ComponentassemblyImpl)assemblyObject);
                ((ComponentassemblyDeployer)
                 assemblyObject
                 .getExtensionManager().getExtensionByName("ccm.Deployer")
                 .getInstance()).tearDownInstantiations(general,(ComponentassemblyImpl)assemblyObject);
                
                etat=(org.omg.Components.Deployment.AssemblyOperations)new AssemblyImplInactive();
                
                am.remove();
            }
            catch(Exception e){e.printStackTrace();}
            
        }
        
        /**
         * Break connection between composant and destroy them
         */	
        public org.omg.Components.Deployment.AssemblyState get_state()
        {
            return org.omg.Components.Deployment.AssemblyState.INSERVICE;
        }
    }
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================


    /**
     * Method to setup the path to the XML file to process
     */     
    public void 
    setupXMLFilePath(String xmlFilePath)
    {		
        this.xmlFilePath=xmlFilePath;
        general=new java.util.Hashtable();
        etat=new AssemblyImplInactive();
    }
    
    /**
     * Method to setup the XML code to process.
     */
    public void 
    setupXML(String xml)
    {		
        this.xml=xml;
        general=new java.util.Hashtable();
        etat=new AssemblyImplInactive();
    }

    public void
    setupAssemblyManager(org.objectweb.openccm.OpenCCM_DCI.AssemblyManager am)
    {
        this.am=am;
    }   
	    

    // ==================================================================
    //
    // Assembly interface methods    
    //
    // ==================================================================

    /*Delegation operation to the state assembly*/
    public void 
    build() 
    throws org.omg.Components.CreateFailure
    {
        etat.build();	
    }
    
    public void 
    tear_down() 
    throws org.omg.Components.RemoveFailure 
    {
        etat.tear_down();	
    }
    
    
    public org.omg.Components.Deployment.AssemblyState 
    get_state()
    {
        return etat.get_state();
    }
   
    
}
