/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/



package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;
import org.ist.coach.DCI.HomeInfo;
import org.objectweb.ccm.CORBA.*;
/**
 * This is the implementation of the HomeFinder and HomeRegistration 
 * interfaces..
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */
public class HomeManagementSegImpl 
       extends  org.objectweb.openccm.OpenCCM_DCI.
                DCIManagerSessionComposition.HomeManagementSeg
{

    /*private class HomeInfo{
     
     public String home_repid;
     public String component_repid;
     public String name;
     public org.omg.Components.CCMHome home;
     
     public HomeInfo(org.omg.Components.CCMHome home,
                     String home_repid,
                     String component_repid,
                     String name){
       this.home=home;                 
       this.home_repid=home_repid;
       this.component_repid=component_repid;
       this.name=name;                        
       }   
    }*/
    
    /**
     ** To store (cookie, HomeInfo) associations.
     **/
    private org.objectweb.ccm.util.Table by_cookies_;

    /**
     ** To store (home_repid, CCMHome) associations.
     **/
    private org.objectweb.ccm.util.Table by_home_repids_;

    /**
     ** To store (component_repid, CCMHome) associations.
     **/
    private org.objectweb.ccm.util.Table by_component_repids_;

    /**
     ** To store (home_name, CCMHome) associations.
     **/
    private org.objectweb.ccm.util.Table by_home_names_;
    
    /**
     **  The reference on the DCIManager component interface
     * */ 
    private org.objectweb.openccm.OpenCCM_DCI.DCIManagerCCM DCI;  


    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     **  The default constructor.
     **/
    public
    HomeManagementSegImpl()
    {
        by_cookies_ = new org.objectweb.ccm.util.Table();
        by_home_repids_ = new org.objectweb.ccm.util.Table();
        by_component_repids_ = new org.objectweb.ccm.util.Table();
        by_home_names_ = new org.objectweb.ccm.util.Table();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    public void setupDCI(){
        DCI=(org.objectweb.openccm.OpenCCM_DCI.DCIManagerCCM)get_context();        
    }
   

    // ==================================================================
    //
    // Methods for the Components::HomeFinder interface.
    //
    // ==================================================================

    //
    // IDL:omg.org/Components/HomeFinder/find_home_by_component_type:1.0
    //
    /**
     **  Finds an home by its managed component type.
     **
     **  @param comp_repid The requested component repository ID.
     **
     **  @return The associated home.
     **
     **  @exception org.omg.Components.HomeNotFound
     **             Thrown if there is no home for the requested
     **             component type.
     **/
    public org.omg.Components.CCMHome
    find_home_by_component_type(String comp_repid)
    throws org.omg.Components.HomeNotFound
    {
    // Search by component repository id.
    Object result = by_component_repids_.get(comp_repid);

    // Check that a CCMHome has been found.
    if(result == null)
    {
        throw new org.omg.Components.HomeNotFound();
    }

    // Return the found CCMHome.
    return (org.omg.Components.CCMHome)result;
    }

    //
    // IDL:omg.org/Components/HomeFinder/find_home_by_home_type:1.0
    //
    /**
     **  Finds an home by its home type.
     **
     **  @param home_repid The requested home repository ID.
     **
     **  @return The associated home.
     **
     **  @exception org.omg.Components.HomeNotFound
     **             Thrown if there is no home for the requested home type.
     **/
    public org.omg.Components.CCMHome
    find_home_by_home_type(String home_repid)
    throws org.omg.Components.HomeNotFound
    {
    // Search by home repository id.
    Object result = by_home_repids_.get(home_repid);

    // Check that a CCMHome has been found.
    if(result == null)
    {
        throw new org.omg.Components.HomeNotFound();
    }

    // Return the found CCMHome.
    return (org.omg.Components.CCMHome)result;
    }

    //
    // IDL:omg.org/Components/HomeFinder/find_home_by_name:1.0
    //
    /**
     **  Finds an home by its name.
     **
     **  @param home_name The requested home name.
     **
     **  @return The associated home.
     **
     **  @exception org.omg.Components.HomeNotFound
     **             Thrown if there is no home for the requested home name.
     **/
    public org.omg.Components.CCMHome
    find_home_by_name(String home_name)
    throws org.omg.Components.HomeNotFound
    {
    // Search by home name.
    Object result = by_home_names_.get(home_name);

    // Check that a CCMHome has been found.
    if(result == null)
    {
        throw new org.omg.Components.HomeNotFound();
    }

    // Return the found CCMHome.
    return (org.omg.Components.CCMHome)result;
    }


    public HomeInfo []
    get_all_homes()
    {                
        java.util.Collection cc=
            by_cookies_.values();                      
        
        return 
            (HomeInfo [])
                cc.toArray(new HomeInfo[cc.size()]);         
                        
    }
    // ==================================================================
    //
    // Methods for the OpenCCM::Components::HomeManager interface.
    //
    // ==================================================================

    //
    // IDL:goal.lifl.fr/OpenCCM/Components/HomeManager/register_home:1.0
    //
    /**
     **  Register a CCMHome.
     **
     **  @param home The CCMHome reference.
     **  @param home_repid The CCMHome Repository Id.
     **  @param component_repid The managed component Repository Id.
     **  @param name The INS name associated to the home.
     **
     **  @return The cookie associated to this registration.
     **/
    public org.omg.Components.Cookie
    register_home(org.omg.Components.CCMHome home,
                  String home_repid,
                  String component_repid,
                  String name)
    {
    

    try{
        // Connect the Home to the DCI
        org.omg.Components.Cookie cookie = DCI.connect_homes(home);
        // Register the home information by the generated cookie.
        by_cookies_.put(cookie,
            new org.ist.coach.DCI.HomeInfo(
                    home_repid,
                    component_repid,
                    home,                    
                    name,
                    "Unknown"));
        // Register the home by its repository Id.
        by_home_repids_.put(home_repid, home);

        // Register the home by its managed component repository Id.
        by_component_repids_.put(component_repid, home);

        // Register the home by its name.
        by_home_names_.put(name, home);
    
        // Return the generated cookie.
        return cookie;
        }
    catch(Exception e){
            System.err.println("Cannot connect home to the DCI");
            return null;
        }
    }

    //
    // IDL:goal.lifl.fr/OpenCCM/Components/HomeManager/unregister_home:1.0
    //
    /**
     **  Unregister a CCMHome.
     **
     **  @param The registration cookie.
     **
     **  @exception ::Components::CookieRequired
     **             Thrown when an invalid cookie is used.
     **/
    public void
    unregister_home(org.omg.Components.Cookie ck)
    throws org.omg.Components.CookieRequired
    {
    if(ck==null)
        throw  new org.omg.Components.CookieRequired();
    // Search the home info by its associated cookie.
    HomeInfo home_info = (HomeInfo)by_cookies_.get(ck);

    // Check that a home info is associated to the cookie.
    if(home_info == null)
    {
        throw new org.omg.Components.CookieRequired();
    }

    // Remove in the home repository id table.
    by_home_repids_.remove(home_info.home_repid);

    // Remove in the component repository id table.
    by_component_repids_.remove(home_info.component_repid);

    // Remove in the home name table.
    by_home_names_.remove(home_info.uid);
    try{
        //disconnect the home from the DCI
        DCI.disconnect_homes(ck);
        }
    catch(Exception e){
            System.err.println("Cannot disconnect home from the DCI");
        }
    }


}
