/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ---------------------------------------------

====================================================================*/

package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;
/**
 * This is the implementation of the NodeManagement facet provide
 * by the NodeManager component.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */
public class NodeManagementSegImpl  
    extends org.objectweb.openccm
            .OpenCCM_DCI.DCIManagerSessionComposition
            .NodeManagementSeg    
    {
        // ==================================================================
        //
        // Internal state.
        //
        // ==================================================================
        // cookie translation table ( TO REWRITE)
        private java.util.Hashtable nodes; 
        //The main DCI component
        private org.objectweb.openccm.OpenCCM_DCI.DCIManagerCCM DCI;        

        // ==================================================================
        //
        // Internal methods.
        //
        // ==================================================================
        // ==================================================================
        //
        // Constructor
        //
        // ==================================================================
        public NodeManagementSegImpl(){
            super();
            nodes=new java.util.Hashtable();
        }
        
        // ==================================================================
        //
        // Public methods.
        //
        // ==================================================================
        /**
         * Setup the DCI
         */
        public void setupDCI(){
            DCI=(org.objectweb.openccm.OpenCCM_DCI.DCIManagerCCM)
                get_context(); 
        }        

        /**
         * Add a node manager.
         *
         * @param node_name The name of the node manager.
         * @param node_manager The reference to the node manager.
         *
         * @return A cookie.
         *
         * WARNING: Need an exception to signal that the node name is already used?
         */
        public void
        register_node(
                      String node_name,
                      org.ist.coach.DCI.NodeManager node_manager)
        {
            try{
                nodes.put(node_name,
                          ((org.objectweb.openccm.OpenCCM_DCI.DCIManagerCCM)
                           get_context())
                          .connect_node_managers(node_manager) ); 
            }
            catch(Exception e){
                e.printStackTrace();                
            }
        }

        /**
         * Remove a previously added node manager.
         *
         * @param cookie The cookie returned when added the node manager.
         *
         * @throw ::Components::CookieRequired
         *        Thrown when the provided cookie is invalid.
         */
        public void 
        deregister_node( String  node_name)
        throws org.ist.coach.DCI.UnknownNode
        {
            try{
                
                ((org.objectweb.openccm.OpenCCM_DCI.DCIManagerCCM)
                 get_context())
                    .disconnect_node_managers((org.omg.Components.Cookie)
                                              nodes.get(node_name));
            }
            catch(Exception e){
                e.printStackTrace();
                throw new org.ist.coach.DCI.UnknownNode();
            }
        }
                             
}
