/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ---------------------------------------------

====================================================================*/

package org.objectweb.openccm.OpenCCM_DCI.cif;

import org.objectweb.openccm.OpenCCM_DCI.*;

/**
 * This is the implementation of the NodeManager component type.
 *
 * This class inherits from the skeleton
 * generated by the OpenCCM's CIF to Java mapping generator.
 *
 * @author <a href="mailto:Frederic.Briclet@lifl.fr">Briclet Frederic</A>
 */

public class NodeManagerImpl
     extends org.objectweb.openccm.OpenCCM_DCI
             .NodeManagerSessionComposition
             .NodeManagerSessionImpl
{
    
    // ==================================================================
    //
    // Internal state.
    //
    // ================================================================== 
    //The node name
    private String nodeName;
    //File path to the NodePropertiesDescriptor
    private String npdFilePath;
    
      
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    // ==================================================================
    //
    // Constructor.
    //
    // ================================================================== 
    public NodeManagerImpl(String npdFilePath){
        //save the file path
        this.npdFilePath=npdFilePath;
    }
  
    // ==================================================================
    //
    // Public methods.
    //
    // ================================================================== 
     /**
     * Container callback to signal that the component is removed.
     *
     * @throw org.omg.Components.CCMException For any problems.*
     */
    public void
    ccm_remove()
    throws org.omg.Components.CCMException
    {
        try{
            ((org.objectweb.openccm.OpenCCM_DCI.NodeManagerCCM)get_context()).
                get_connection_node_management().deregister_node(nodeName); 
            (new org.objectweb.ccm.Deployment.ShutDownThread()).start();
        }
        catch(org.ist.coach.DCI.UnknownNode e){
            e.printStackTrace();
            throw new org.omg.Components.CCMException();
        }
    }

    /**
     * Completes the component configuration.
     *
     * @exception org.omg.Components.InvalidConfiguration
     *            Thrown if the configuration is invalid.
     */
    public void
    configuration_complete()
    throws org.omg.Components.InvalidConfiguration
    {
      try{
          //setup the node
          ((NodeInformationSegImpl)
              obtain_executor("node_information")).setTree(npdFilePath);      
          
          org.omg.Components.SessionContext sc=
              (org.omg.Components.SessionContext)get_context();
          //register the node to the DCI
          ((org.objectweb.openccm.OpenCCM_DCI.NodeManagerCCM)get_context()).
              get_connection_node_management()
              .register_node(nodeName,org.objectweb.openccm.
                             OpenCCM_DCI.NodeManagerHelper
                             .narrow(sc.get_CCM_object()));   
      }
      catch(Exception e){
          e.printStackTrace();
      }      
    }
    
    // ==================================================================
    //
    // Accessor methods.
    //
    // ================================================================== 
    public void node_name(String node_name){
        this.nodeName=node_name;
    }

    public String node_name(){
     return nodeName;   
    }
    
    
}
