/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.Components;

/** The console's imports. */
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultEntry;

/** The CCM's imports */
import org.omg.Components.CCMObject;
import org.omg.Components.ComponentPortDescription;
import org.omg.Components.FacetDescription;
import org.omg.Components.ReceptacleDescription;
import org.omg.Components.ConsumerDescription;
import org.omg.Components.EmitterDescription;
import org.omg.Components.PublisherDescription;
import org.omg.Components.PortDescription;

/**
 * Basic implementation of the Context interface
 * for managing Components.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class ComponentContext 
  implements Context, Wrapper{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The Component. */
    private CCMObject component_ = null;

    // ==================================================================
    //
    // No Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    public Entry[] getEntries() {

        ComponentPortDescription cpd = component_.get_all_ports();
        java.util.List v = new java.util.Vector();

        FacetDescription[] fd = cpd.facets;
        for (int i = 0; i < fd.length; i++)
            v.add(new DefaultEntry(fd[i], new DefaultName(fd[i].name), this));

        ConsumerDescription[] cd = cpd.consumers;
        for (int i = 0; i < cd.length; i++)
            v.add(new DefaultEntry(cd[i], new DefaultName(cd[i].name), this));

        ReceptacleDescription[] rd = cpd.receptacles;
        for (int i = 0; i < rd.length; i++) {
            ReceptacleContainer rc = new ReceptacleContainer();
            rc.setComponent(component_);
            rc.setReceptacle(rd[i]);
            v.add(new DefaultEntry(rc, new DefaultName(rd[i].name), this));
        }

        EmitterDescription[] ed = cpd.emitters;
        for (int i = 0; i < ed.length; i++)
            v.add(new DefaultEntry(ed[i], new DefaultName(ed[i].name), this));

        PublisherDescription[] pd = cpd.publishers;
        for (int i = 0; i < pd.length; i++) {
            PublisherContainer pc = new PublisherContainer();
            pc.setComponent(component_);
            pc.setPublisher(pd[i]);
            v.add(new DefaultEntry(pc, new DefaultName(pd[i].name), this));
        }

        return (Entry[]) v.toArray(new Entry[0]);
    }

    // ==================================================================
    //
    // Public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public void setWrapped(Object object) {
        component_ = (CCMObject) object;
    }

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public Object getWrapped() {
        return component_;
    }

}
