/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.CosNaming;

/** The administration console's API imports */
import org.objectweb.util.browser.api.MenuItem;
import org.objectweb.util.browser.api.MenuItemTreeView;
import org.objectweb.util.browser.api.TreeView;
import org.objectweb.util.browser.gui.api.DialogAction;
import org.objectweb.util.browser.gui.api.DialogBox;
import org.objectweb.util.browser.gui.lib.DefaultDialogBox;
import org.objectweb.util.browser.gui.lib.LabelBox;

/** The CORBA's imports */
import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.NameComponent;

/**
 * This action binds a new context into the NameService.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 *
 * @version 0.1
 */
public class BindNewContextAction 
    implements MenuItem, DialogAction {

    //==================================================================
    //
    // Internal states.
    //
    //==================================================================

    /** The dialog box */
    protected DialogBox dialog_;
    
    /** The associated NamingContext */
    protected NamingContextExt namingContext_ = null;

    /** The name of the object */
    protected LabelBox name_ = null;

    //==================================================================
    //
    // No constructor.
    //
    //==================================================================

    //==================================================================
    //
    // No internal method.
    //
    //==================================================================
 
    //==================================================================
    //
    // Public methods for MenuItem interface.
    //
    //==================================================================

    /**
     * @see org.objectweb.util.browser.api.MenuItem#getStatus(org.objectweb.util.browser.api.TreeView)
     */
    public int getStatus(TreeView treeView) {
        return MenuItem.ENABLED_STATUS;
    }

    /**
     * The action to execute
     */
    public void actionPerformed(MenuItemTreeView e) {
        namingContext_ = (NamingContextExt) e.getSelectedObject();
        dialog_ = new DefaultDialogBox("Binds a new context into the NameService");      
        name_ = new LabelBox("Context name");
        dialog_.addElementBox(name_);
        dialog_.setValidateAction(this);
        dialog_.show();
    }

    //==================================================================
    //
    // Public methods for DialogAction interface.
    //
    //==================================================================

    /**
     * Executes an action
     */
    public void executeAction() throws Exception {
        NameComponent[] names = namingContext_.to_name(name_.getLabel());
        NamingContext namingContext = namingContext_;
        for (int i = 0 ; i < names.length ; i++) {
            NameComponent[] currentNameComponent = new NameComponent[]{names[i]};
            NamingContext newNamingContext = null;
            try {
                newNamingContext = namingContext.bind_new_context(currentNameComponent);
            } catch(org.omg.CosNaming.NamingContextPackage.AlreadyBound e){
                // Ignore : a namingContext with the same name already exists !
                newNamingContext = org.omg.CosNaming.NamingContextExtHelper.narrow(namingContext.resolve(currentNameComponent));
            }
            namingContext = newNamingContext;
        }    
    }
}