/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.CosTrading;

/** The administration console's API imports */
import org.objectweb.util.browser.api.Context;
import org.objectweb.util.browser.api.Wrapper;
import org.objectweb.util.browser.api.Entry;
import org.objectweb.util.browser.core.naming.DefaultName;
import org.objectweb.util.browser.core.naming.DefaultEntry;

/** The OMG Trading Service imports. */
import org.omg.CosTrading.Link;
import org.omg.CosTrading.LinkPackage.LinkInfo;

/**
 * Basic implementation of the Context interface
 * for managing trader Links. 
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class LinkContext
  implements Context, Wrapper
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The link to navigate. */
    private Link link_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for interface Context.
    //
    // ==================================================================

    public Entry[] getEntries() {

        String[] names = link_.list_links();
        Entry[] values = new Entry[names.length];

        for (int i = 0; i < names.length; i++) {
            try {
                LinkInfo link = link_.describe_link(names[i]);
                values[i] = new DefaultEntry(link, 
                                             new DefaultName(names[i]), 
                                             this);
            } catch (org.omg.CosTrading.LinkPackage.IllegalLinkName ex) {
                // cannot happened !! 
            } catch (org.omg.CosTrading.LinkPackage.UnknownLinkName ex) {
                // cannot happened !! 
            }
        }
        return values;        
    }

    // ==================================================================
    //
    // Additional public methods for Wrapper interface.
    //
    // ==================================================================

    /**
     * Fix the wrapped object
     *
     * @param object The object to wrap
     *
     */
    public void setWrapped(Object object) {
        link_ = (Link) object;
    }

    /**
     * Gets the wrapped object
     *
     * @return The wrapped object
     *
     */
    public Object getWrapped() {
        return link_;
    }

}
