/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.CosTrading.gui;

import org.omg.CosTrading.Admin;

/**
 * An abstract class for any TextAction Listener which will perform a
 * CosTrading admin action when an event occured.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
*/
public abstract class AbstractTextActionListener implements java.awt.event.ActionListener {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The CosTrading admin reference. */
    protected Admin adm_; 

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor.
     *
     * @param adm The CosTrading admin reference.
     */
    public AbstractTextActionListener(Admin adm) {adm_ = adm;} 

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Must be completed by the admin action to perform.
     */
    protected abstract void doAdminAction(java.awt.event.ActionEvent e);

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    public void actionPerformed(java.awt.event.ActionEvent e) {
        doAdminAction(e);
    }
}

