/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.CosTrading.gui;

/** The java API's imports */
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.SwingConstants;

import java.awt.Component;
import java.awt.Dimension;

/** The OMG Trading Service imports. */
import org.omg.CosTrading.FollowOption;

/**
 * A Box to display a CosTrading FollowOption in a combo box.  This
 * class also provides a way to register a listener in order to be
 * notified of any change in the combo box.
 * 
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class FollowOptionBox extends Box {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The combo box. */
    protected JComboBox options_;

    /** The label of the entry */
    protected JLabel fieldLabel_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** 
     * Default constructor.
     *
     * @param label The label placed behind the combo box.
     */
    public FollowOptionBox(String label) {
        super(BoxLayout.X_AXIS);
        add(Box.createHorizontalGlue());
        fieldLabel_ = new JLabel(label, SwingConstants.RIGHT);
        fieldLabel_.setAlignmentX(Component.RIGHT_ALIGNMENT);
        fieldLabel_.setAlignmentY(Component.CENTER_ALIGNMENT);
        add(fieldLabel_);
        add(Box.createHorizontalStrut(5));
        options_ = new JComboBox(new String[] {"local_only", "if_no_local", "always"});
        options_.setPreferredSize(new Dimension(225, 20));
        options_.setMaximumSize(new Dimension(225, 20));
        add(options_);
    }

    /** 
     * Default constructor.
     *
     * @param label The label placed behind the combo box.
     * @param init The initial value of the combo box.
     */
    public FollowOptionBox(String label, FollowOption init) {
        this(label);
        options_.setSelectedIndex(optionToIndex(init));
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /** 
     * Returns the index of the follow option in the combo box.
     *
     * @param o The follow option.
     *
     * @return The index of the given option.
     */
    protected int optionToIndex(FollowOption o) {
        if (o.value() == FollowOption._local_only) return 0;
        if (o.value() == FollowOption._if_no_local) return 1;
        return 2;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /** 
     * Returns the specified option.
     *
     * @return The option selected in the combo box.
     */
    public FollowOption getOption() {
        String option = (String)options_.getSelectedItem();
        if (option.equals("local_only")) return FollowOption.local_only;
        if (option.equals("if_no_local")) return FollowOption.if_no_local;
        return FollowOption.always;
    }

    /**
     * Adds a listener on the combo box.
     *
     * @param listener The listener to add in the combo box.
     */
    public void addItemListener(java.awt.event.ItemListener listener) {
        options_.addItemListener(listener);
    }

    // ==================================================================
    //
    // Public methods of Box (since j2sdk-1.4.1 only).
    //
    // ==================================================================

    /**
     * Sets the tool tip text on each JComponent of the box.
     *
     * @param text The tool tip text to set.
     */
    public void setToolTipText(String text) {
        options_.setToolTipText(text);
        fieldLabel_.setToolTipText(text);
    }

    /**
     * Gets the tool tip text set on each JComponent of the box.
     *
     * @return The tool tip text used.
     */
    public String getToolTipText() {
        return options_.getToolTipText();
    }
}
