/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s):Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.CosTrading.gui;

/** The Java API's imports */
import java.awt.Color;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.border.TitledBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.JScrollPane;

/** OMG Trading Service imports. */
import org.omg.CosTrading.ProxyPackage.ProxyInfo;

/**
 * Panel used to view content of
 * <code>org.omg.CosTrading.ProxyPackage.ProxyInfo</code>
 * instances. Objects of this type are obtained by the
 * <code>list_proxies</code> method of the
 * <code>org.omg.CosTrading.Admin</code> interface. This panel should
 * be completed with displaying proxy info with policies to pass on.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class ProxyInfoPanel extends DefaultNodeViewPanel {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The Proxy offer to view. */
    protected ProxyInfo proxyInfo_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor.
     */
    public ProxyInfoPanel() {
        setBackground(Color.white);
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for class DefaultNodeViewPanel.
    //
    // ==================================================================

    /**
     * Creates the content of the Panel.
     */
    public void onSetTreeView() {
        if (getTreeView() != null) {
            proxyInfo_ = (ProxyInfo)getTreeView().getSelectedObject();
            if (proxyInfo_ != null) {
                
                // Creates Service type label
                JLabel st = new JLabel("Service Type: " + proxyInfo_.type); 
                
                // Creates Properties table
                TableModel dataModel = new OfferPropsTableModel(proxyInfo_.properties);
                JTable table = new JTable(dataModel);
                
                // Creates table related scroll pane
                JScrollPane scroll = new JScrollPane(table);
                scroll.setBorder(new TitledBorder(null, " Properties ",TitledBorder.CENTER,TitledBorder.TOP));
                
                // Creates match all label
                JLabel match = new JLabel("If match all: " + proxyInfo_.if_match_all); 
                
                // Creates recipe label
                JLabel recipe = new JLabel("Constraint recipe: " + proxyInfo_.recipe); 
                
                // Adds all labels to the box
                Box box = Box.createVerticalBox();
                box.add(st);
                box.add(Box.createVerticalStrut(10));
                box.add(scroll);
                box.add(Box.createVerticalStrut(10));
                box.add(match);
                box.add(Box.createVerticalStrut(10));
                box.add(recipe);
                
                add(box);
            }
        }
    }
}
