/*===========================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s):Sylvain Leblanc.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.openccm.browser.CosTrading.gui;

/** The Java API's imports */
import javax.swing.JPanel;
import java.awt.Color;
import java.lang.reflect.Method;
import java.lang.reflect.Field;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.border.TitledBorder;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.TableModel;
import java.awt.Dimension;

/** OMG Trading Service imports. */
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.TypeStruct;
import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.PropStruct;

/**
 * Panel used to display the content of a trading service type. 
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public class ServiceTypePanel extends DefaultNodeViewPanel {

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    protected TypeStruct type_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor.
     */
    public ServiceTypePanel() {
        setBackground(Color.white);
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Creates a Panel containing the service type properties.
     *
     * @return A Panel containing the service type properties.
     */
    protected JPanel addProperties() {
        JPanel propertiesPanel = new JPanel();
        propertiesPanel.setBackground(Color.white);
        propertiesPanel.setBorder(new TitledBorder(null," Properties ",TitledBorder.CENTER,TitledBorder.TOP));
        propertiesPanel.setLayout(new java.awt.GridLayout(1, 1));

        Box box = Box.createVerticalBox();

        box.add(Box.createVerticalGlue());
        // Creates a JTable for properties
        TableModel dataModel = new ServiceTypePropsTableModel(type_.props);
        JTable table = new JTable(dataModel);
        table.setBackground(Color.white);
        table.setPreferredScrollableViewportSize(table.getMaximumSize());

        JScrollPane scrollpane = new JScrollPane(table);
        scrollpane.setBackground(Color.white);
       
        box.add(scrollpane);
        box.add(Box.createVerticalGlue());

        propertiesPanel.add(box);

        return propertiesPanel;
    }

    /**
     * Creates a Panel containing the service type interface.
     *
     * @return A Panel containing the service type interface.
     */
    protected JPanel addInterface() {
        JPanel itfPanel = new JPanel();
        itfPanel.setBackground(Color.white);
        itfPanel.setBorder(new TitledBorder(null," IDL interface ",TitledBorder.CENTER,TitledBorder.TOP));

        Box box = Box.createVerticalBox();
        String if_name = type_.if_name;
        box.add(new ItfGUI(if_name));
        itfPanel.add(box);

        return itfPanel;
    }

    /**
     * Creates a Panel containing the service type super types.
     *
     * @return A Panel containing the service type super types.
     */
    protected JPanel addSuperTypes() {
        JPanel stPanel = new JPanel();
        stPanel.setBackground(Color.white);
        stPanel.setBorder(new TitledBorder(null," Super types ",TitledBorder.CENTER,TitledBorder.TOP));

        Box box = Box.createVerticalBox();
        String[] st = type_.super_types;
        box.add(new SuperTypesGUI(st));
        stPanel.add(box);

        return stPanel;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Creates the content of the Panel.
     */
    public void onSetTreeView() {
        if (getTreeView() != null) {
            type_ = (TypeStruct)getTreeView().getSelectedObject();
            if (type_ != null) {
                add(addSuperTypes());
                add(addInterface());
                add(addProperties());
            }
        }
    }
}














