/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy.
Contributor(s):  ______________________________________.

====================================================================*/
package org.objectweb.openccm.browser.menu;

/** The Java API's imports */
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

/**
 * This class provides a JFileChooser.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class JFileChooserSingleton {

    /** The JFileChooser instance */
    protected static JFileChooser instance_ = null;

    /** The JarFileFilter instance */
    protected static FileFilter jarFileFilter = new JarFileFilter();

    /** The JarFileFilter instance */
    protected static FileFilter xmlFileFilter = new XMLFileFilter();

    /** The IORFileFilter instance */
    protected static FileFilter iorFileFilter = new IORFileFilter();

    /**
     * Initialize the instance
     */
    protected static void init() {
        instance_ = new JFileChooser();
    }

    /**
     * Provides a JFileChooser with a specific FileFilter
     * The default dialogTitle is Open and the defaulf fileSelectionMode is JFileChooser.FILES_ONLY.
     * @param fileFilterType The FileFilter type.
     *  <ul>
     *    <li>OpenCCMBrowserConstants.JAR_FILE_CHOOSER</li>
     *    <li>OpenCCMBrowserConstants.XML_FILE_CHOOSER</li>
     *    <li>OpenCCMBrowserConstants.IOR_FILE_CHOOSER</li>
     *    <li>OpenCCMBrowserConstants.DIRECTORY_FILE_CHOOSER</li>
     *  </ul>
     * @return The appropriate JFileChooser
     */
    public static JFileChooser getInstance(int fileFilterType) {
        return getInstance(fileFilterType, "Open", JFileChooser.FILES_ONLY);    
    }

    /**
     * Provides a JFileChooser with a specific FileFilter
     * The default dialogTitle is Open and the defaulf fileSelectionMode is JFileChooser.FILES_ONLY.
     * @param fileFilterType The FileFilter type.
     *  <ul>
     *    <li>OpenCCMBrowserConstants.JAR_FILE_CHOOSER</li>
     *    <li>OpenCCMBrowserConstants.XML_FILE_CHOOSER</li>
     *    <li>OpenCCMBrowserConstants.IOR_FILE_CHOOSER</li>
     *    <li>OpenCCMBrowserConstants.DIRECTORY_FILE_CHOOSER</li>
     *  </ul>
     * @param fileSelectionMode The file selection mode
     *  <ul>
     *    <li>JFileChooser.FILES_ONLY</li>
     *    <li>JFileChooser.DIRECTORIES_ONLY</li>
     *    <li>JFileChooser.FILES_AND_DIRECTORIES</li>
     *  </ul>
     * @return The appropriate JFileChooser
     */
    public static JFileChooser getInstance(int fileFilterType, int fileSelectionMode) {
        return getInstance(fileFilterType, "Open", fileSelectionMode);    
    }

    /**
     * Gives a JFileChooser with a specific FileFilter
     * @param fileFilterType The FileFilter type.
     *  <ul>
     *    <li>OpenCCMBrowserConstants.JAR_FILE_CHOOSER</li>
     *    <li>OpenCCMBrowserConstants.XML_FILE_CHOOSER</li>
     *    <li>OpenCCMBrowserConstants.IOR_FILE_CHOOSER</li>
     *    <li>OpenCCMBrowserConstants.DIRECTORY_FILE_CHOOSER</li>
     *  </ul>
     * @param dialogTitle The title of the JFileChooser box.
     * @param fileSelectionMode The file selection mode
     *  <ul>
     *    <li>JFileChooser.FILES_ONLY</li>
     *    <li>JFileChooser.DIRECTORIES_ONLY</li>
     *    <li>JFileChooser.FILES_AND_DIRECTORIES</li>
     *  </ul>
     * @return The appropriate JFileChooser
     */
    public static JFileChooser getInstance(int fileFilterType, String dialogTitle, int fileSelectionMode) {
        File f = null;
        if (instance_ != null)
            f = instance_.getCurrentDirectory();
        init();
        instance_.setCurrentDirectory(f);
        instance_.setDialogTitle(dialogTitle);
        try{
            instance_.setFileSelectionMode(fileSelectionMode);
        }catch(IllegalArgumentException e){}
        if (fileFilterType == OpenCCMBrowserConstants.JAR_FILE_CHOOSER)
            instance_.setFileFilter(jarFileFilter);
        else if (fileFilterType == OpenCCMBrowserConstants.XML_FILE_CHOOSER)
            instance_.setFileFilter(xmlFileFilter);
        else if (fileFilterType == OpenCCMBrowserConstants.IOR_FILE_CHOOSER)
            instance_.setFileFilter(iorFileFilter);
        else if (fileFilterType == OpenCCMBrowserConstants.DIRECTORY_FILE_CHOOSER)
            instance_.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        return instance_;        
    }

}
