/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jrme Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.gui.panel;

/** The Java API'imports */
import javax.swing.JPanel;
import javax.swing.JButton;
import java.awt.Dimension;
import java.awt.Color;
import java.lang.reflect.Method;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.border.TitledBorder;
import javax.swing.BoxLayout;
import javax.swing.Box;
import java.awt.Component;

/** The Console's imports */
import org.objectweb.util.browser.gui.common.TreeView;

/**
 * This class represents a Method
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class MethodGUI extends Box {

    /** The view on the tree */
    protected TreeView treeView_;

    /** The object which is represented */
    protected Object object_;

    /** The associated method */
    protected Method method_;

    /** The parameters of the method */
    protected ParameterGUI[] parametersGUI_;

    /**
     * 
     * @return the box to add
     */
    protected Box getHeader() {
        Box header = Box.createHorizontalBox();
        //header.add(Box.createHorizontalGlue());
        //header.add(Box.createHorizontalStrut(10));
        JButton button = new JButton("Invoke");
        button.setAlignmentX(Component.LEFT_ALIGNMENT);
        button.setPreferredSize(new Dimension(90, 20));
        button.addActionListener(new InvokeAction());
        header.add(button);
        //header.add(Box.createHorizontalGlue());
        return header;
    }

    /** 
     * Constructs a JPanel containing a representation of the method
     * @param method The method to represent
     * @param treeView A view on the tree
     */
    public MethodGUI(Method method, TreeView treeView) {
        super(BoxLayout.X_AXIS);

        method_ = method;
        treeView_ = treeView;
        object_ = treeView_.getHandleObject();

        Class[] args = method_.getParameterTypes();

        JPanel panel = new JPanel();
        panel.setBackground(Color.white);
        panel.setBorder(new TitledBorder(null,getUMLRepresentation(),TitledBorder.LEFT,TitledBorder.TOP));

        Box box = Box.createVerticalBox();
        box.add(getHeader());

        parametersGUI_ = new ParameterGUI[args.length];
        for (int i = 0; i < parametersGUI_.length; i++) {
            parametersGUI_[i] = new ParameterGUI(args[i]);
            box.add(parametersGUI_[i]);
        }

        panel.add(box);

        add(panel);
    }

    /**
     * Returns the name without the package's name
     * @return the string
     */
    protected String getNameWithoutPackage(String type) {
        int lastIndex = type.lastIndexOf('.');
        if (lastIndex != -1)
            return type.substring(lastIndex + 1);
        return type;
    }

    /**
     * Returns an UML representation of a method
     * 
     * @return The UML representation
     */
    public String getUMLRepresentation() {
        StringBuffer str = new StringBuffer();
        str.append(method_.getName() + "(");
        Class[] args = method_.getParameterTypes();
        for (int j = 0; j < args.length; j++) {
            str.append(getNameWithoutPackage(args[j].getName()));
            if (j != args.length - 1)
                str.append(", ");
        }
        str.append(") : " + method_.getReturnType().getName());
        Class[] exceptions = method_.getExceptionTypes();
        if (exceptions.length != 0) {
            str.append("  throws ");
            for (int j = 0; j < exceptions.length; j++) {
                str.append(getNameWithoutPackage(exceptions[j].getName()));
                if (j != exceptions.length - 1)
                    str.append(", ");
            }
        }
        return str.toString();
    }

    /** 
     * Use to execute the method invocation
     */
    protected class InvokeAction implements ActionListener {

        public void actionPerformed(ActionEvent e) {
            Object[] args = new Object[parametersGUI_.length];
            for (int i = 0; i < parametersGUI_.length; i++) {
                try {
                    args[i] =
                        new Integer(parametersGUI_[i].getParameterValue());
                } catch (java.lang.NumberFormatException ex) {
                    args[i] = parametersGUI_[i].getParameterValue();
                }
            }
            try {
                method_.invoke(object_, args);
                treeView_.refresh();
            } catch (java.lang.IllegalAccessException ex) {
                // ignore
            } catch (java.lang.IllegalArgumentException ex) {
                // ignore
            } catch (java.lang.reflect.InvocationTargetException ex) {
                // ignore
            }
        }

    }

}
