/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.naming.wrappers.common;

/** To use the Name interface. */
import org.objectweb.util.browser.naming.api.Name;

/** To use the Context interface. */
import org.objectweb.util.browser.naming.api.Context;

/**
 * Abstract basic implementation of the Entry interface.
 *
 * This only implements toStringBuffer and equals methods.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */

abstract public class AbstractEntry
    extends AbstractObject
    implements org.objectweb.util.browser.naming.api.Entry {
    // ==================================================================
    //
    // No internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // No constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for class Object.
    //
    // ==================================================================

    /**
     * Tests if a given object is equals to the target entry.
     *
     * @param obj The given object.
     *
     * @return
     *  true   if the given object is the same object of the target name,
     *      or if it is an Entry instance and the value, name, and context
     *         of the given entry are equals to the value, name, and context
     *         of the target entry;
     *  false otherwise.
     *
     * @postcondition
     *     result == (obj == this)
     *  or (     obj instanceof Entry
     *       and getOWValue().equals(((Entry)obj).getOWValue())
     *       and getOWName().equals(((Entry)obj).getOWName())
     *       and getOWContext().equals(((Entry)obj).getOWContext()) );
     */
    public boolean equals(Object obj) {
        // TODO: Must be fully implemented!!!
        return super.equals(obj);
    }

    // ==================================================================
    //
    // Public methods for class AbstractObject.
    //
    // ==================================================================

    /**
     * Appends internal state representation into a given string buffer.
     *
     * This method does not update the target object.
     *
     * @param sb The string buffer where pairs <name,value> are appended.
     *
     * @postcondition isNotModified();
     */
    public void toStringBuffer(StringBuffer sb) {
        sb.append("value=");
        sb.append(getOWValue());
        sb.append(",name=");
        sb.append(getOWName());
        sb.append(",context=");
        sb.append(getOWContext());
    }

    // ==================================================================
    //
    // Public methods for interface Entry.
    //
    // ==================================================================

    /**
     * Gets the value contained in the target entry.
     *
     * This method does not update the value contained in the target entry.
     *
     * @return The value contained in the target entry.
     *
     * @postcondition return == getOWValue();
     */
    abstract public Object getOWValue();

    /**
     * Gets the name of the target entry, in the context it belongs to.
     *
     * This method does not update the name of the target entry.
     *
     * @return The name of the target entry, in the context it belongs to.
     *
     * @postcondition return == getOWName();
     */
    abstract public Name getOWName();

    /**
     * Gets the context containing the target entry,
     * if any, null otherwise.
     *
     * This method does not update the context containing
     * the target entry.
     *
     * @return The context containing the target entry,
     *         if any, null otherwise..
     *
     * @postcondition return == getOWContext();
     */
    abstract public Context getOWContext();
}
