/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.naming.wrappers.common;

/** The console's imports */
import org.objectweb.util.browser.naming.api.ContextContainerFactory;
import org.objectweb.util.browser.naming.api.ContextContainer;
import org.objectweb.util.browser.naming.api.Decoder;

/**
 * Basic implementation for ContextContainer factories.
 * It adds an interceptor which decode the object to add
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class DefaultContextContainerFactory
    implements ContextContainerFactory {

    /** The decoder */
    protected Decoder decoder_ = null;

    /**
     * Default Constructor
     * @param decoder The first link of the chain of decoder
     */
    public DefaultContextContainerFactory(Decoder decoder) {
        decoder_ = decoder;
    }

    /**
     * Creates a new ContextContainer.
     *
     * @return The created ContextContainer.
     */
    public ContextContainer create() {
        ContextContainer cc = new DefaultContextContainer();
        if (decoder_ != null) {
            ContextContainerDecoderManager interceptor = new ContextContainerDecoderManager();
            interceptor.setDecoder(decoder_);
            interceptor.setDelegate(cc);
            return interceptor;
        }
        return cc;
    }

}
