/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.naming.wrappers.common;

/** The console's imports */
import org.objectweb.util.browser.naming.api.Context;
import org.objectweb.util.browser.naming.api.Entry;
import org.objectweb.util.browser.naming.api.EntryIterator;
import org.objectweb.util.browser.naming.api.Decoder;
import org.objectweb.util.browser.naming.api.DecoderFactory;

/**
 * Factory for decoder object. 
 * For example, a decoder is used to transform a stub reference into another reference more specific.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class DefaultDecoderFactory 
    implements DecoderFactory {
    	
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Creates a decoder manager with the appropriate instances of decoder
     * @param factoryConfig The configuration of the chain of decoder (list of decoder to instanciate)
     */
    public Decoder create(Context factoryConfig) {
        Decoder decoder = null;
        if (factoryConfig != null) {
            EntryIterator it = factoryConfig.getOWEntries();
            while (it.hasNext()) {
                Entry entry = it.next();
                Class c = (Class) entry.getOWValue();
                try {
                    if (Decoder.class.isAssignableFrom(c)) {
                        Decoder newDecoder = (Decoder) c.newInstance();
                        newDecoder.setNext(decoder);
                        decoder = newDecoder;
                    }
                } catch (InstantiationException e1) {
                    System.out.println("Error : " + c.getName() + " : Instantiation exception !");
                } catch (IllegalAccessException e1) {
                    System.out.println("Error : " + c.getName() + " : Illegal access exception !");
                }
            }
        }
        return decoder;
    }

}
