/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2003 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Jerome Moroy.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.xmlreader.decoder;

/** To use the generated class */
import org.objectweb.util.browser.browserConfig.BrowserUnmarshaller;
import org.objectweb.util.browser.browserConfig.Browser;
import org.objectweb.util.browser.browserConfig.Include;
import org.objectweb.util.browser.browserConfig.Node;
import org.objectweb.util.browser.browserConfig.Icon;
import org.objectweb.util.browser.browserConfig.Wrapper;
import org.objectweb.util.browser.browserConfig.Panel;
import org.objectweb.util.browser.browserConfig.Menu;
import org.objectweb.util.browser.browserConfig.Item;
import org.objectweb.util.browser.browserConfig.Jar;

/** The console's imports */
import org.objectweb.util.browser.naming.api.Context;
import org.objectweb.util.browser.naming.api.ContextContainer;
import org.objectweb.util.browser.naming.api.Decoder;
import org.objectweb.util.browser.naming.api.SpecificContextFactory;
import org.objectweb.util.browser.naming.api.Entry;
import org.objectweb.util.browser.naming.wrappers.common.DefaultEntry;
import org.objectweb.util.browser.naming.wrappers.common.DefaultName;
import org.objectweb.util.browser.naming.wrappers.common.DefaultContextContainer;
import org.objectweb.util.browser.naming.wrappers.common.DefaultContextFactory;
import org.objectweb.util.browser.api.BrowserProperty;
import org.objectweb.util.browser.api.ContextProperty;
import org.objectweb.util.browser.api.IconFactory;
import org.objectweb.util.browser.gui.core.ClassResolver;
import org.objectweb.util.browser.gui.icon.DefaultIconFactory;
import org.objectweb.util.browser.gui.panel.DefaultPanelFactory;
import org.objectweb.util.browser.gui.panel.NodeViewPanel;
import org.objectweb.util.browser.api.PanelFactory;
import org.objectweb.util.browser.api.MenuFactoryConfiguration;
import org.objectweb.util.browser.gui.popup.DefaultMenuFactory;
import org.objectweb.util.browser.gui.popup.ItemProperty;

/** The java API's imports */
import java.io.File;
import java.io.InputStream;
import java.io.FileInputStream;
import java.util.List;
import java.util.Iterator;
import java.util.Vector;
import java.net.URL;

/**
 * Parses a Java object which represents an XML Configuration. 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jerome Moroy</a>
 * @version 0.1
 */
public class BrowserXMLDecoder 
    implements XMLDecoder, BrowserProperty {
	
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The context which contains the XML properties */
    protected ContextContainer context_;

    /** The context property to use */
    protected ContextProperty contextProperty_;

    /** The decoder to use */
    protected Decoder decoder_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor
     */
    public BrowserXMLDecoder() {
        context_ = new DefaultContextContainer();

    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Add an Include element into the context
     *
     * @param include The Include element to interpret
     */
    protected void addInclude(Include include) {
        try {
            URL url = new URL(include.getUrl());
            buildBrowser(url.openStream());
        } catch (java.net.MalformedURLException e) {
            if (include.getUrl() != null && !include.getUrl().equals(""))
                System.out.println(include.getUrl() + " : Malformed URL !");
            else
                System.out.println("null URL exists in the XML file !");
        } catch (java.io.FileNotFoundException e) {
            System.out.println(include.getUrl() + " : File not found !");
        } catch (java.io.IOException e) {
            System.out.println(e.getMessage());
        }
    }

    /**
     * Creates an Entry.
     *
     * @param value The value of the entry to create.
     * @param id The id of the name of the entry to create.
     *
     * @return A new Entry.
     */
    protected Entry createEntry(String id, Object value) {
        return new DefaultEntry(value, new DefaultName(id), null);
    }

    /**
     * Tries to load the jar file associated to the given node.
     * @param node The research key
     */
    protected void loadJar(Node node) {
        if (node != null) {
            List jarList = node.getJarList();
            List urlList = new Vector();
            for (Iterator i = jarList.iterator(); i.hasNext();) {
                Jar jar = (Jar) i.next();
                String jarFile = jar.getUrl();
                try {
                    urlList.add(new URL(jarFile));
                } catch (java.net.MalformedURLException e) {
                    System.out.println(jarFile + " : Malformed URL !");
                }
            }
            if (!urlList.isEmpty()) {
                ClassResolver.addContext((URL[]) urlList.toArray(new URL[0]));
            }
        }
    }

    /**
     * Looks for the node with the given id
     * @param id The target of the node
     * @return The associated node (or null)
     */
    protected Node getNode(String id) {
        return (Node) context_.getOWLocalValue(id);
    }

    /** 
     * Parses the XML file and builds the appropriate context
     */
    protected void buildContext(Browser browser) {
        if (browser != null) {
            List includeList = browser.getIncludeList();
            for (Iterator i = includeList.iterator(); i.hasNext();) {
                Include include = (Include) i.next();
                addInclude(include);
            }
            List nodeList = browser.getNodeList();
            for (Iterator i = nodeList.iterator(); i.hasNext();) {
                Node node = (Node) i.next();
                loadJar(node);
                // Add an entry for java class name
                String nodeJavaClass = node.getJavaClass();
                if (nodeJavaClass != null && !nodeJavaClass.equals(""))
                    context_.addEntry(nodeJavaClass, node);
                // Add an entry for idl type
                String nodeIdlType = node.getIdl();
                if (nodeIdlType != null && !nodeIdlType.equals(""))
                    context_.addEntry(nodeIdlType, node);
            }

        }
    }

    /**
     * Builds the Browser object and builds the context if the created Browser is not null.
     * @param stream A stream opened on the file containing the data defining the Browser
     */
    protected void buildBrowser(InputStream stream) {
        try {
            // We can't validate with DTD at this time (no way to fix the DTD file)
            Browser browser = BrowserUnmarshaller.unmarshal(stream);
            if (browser != null)
                buildContext(browser);
        } catch (java.io.IOException e) {
            System.out.println(e.getMessage());
        }
    }

    /**
     * Builds the Browser object and builds the context if the created Browser is not null.
     * @param file The file containing the data defining the Browser
     */
    protected void buildBrowser(File file) {
        try {
            buildBrowser(new FileInputStream(file));
        } catch (java.io.FileNotFoundException e) {
            System.out.println("Error : " + file + " : File not found !");
        }
    }

    // ==================================================================
    //
    // Public methods of XMLDecoder interface.
    //
    // ==================================================================

    /**
     * Fixes the file which contains the properties.
     * @param path The path containing the XML file(s) containing the configuration 
     * we want to abtain under a Context view (It can be a single file or a directory)
     */
    public void setPropertyFile(File path) {
        if (path != null) {
            if (path.isDirectory()) {
                File[] files = path.listFiles(new XMLFileFilter());
                for (int i = 0; i < files.length; i++) {
                    buildBrowser(files[i]);
                }
            } else {
                buildBrowser(path);
            }
        } else
            System.out.println("The path param is null ! ");
    }

    /**
     * Fixes an array of files (or directories) which contains the properties
     * @param files The path containing the XML file(s) containing the configuration 
     * we want to abtain under a Context view (Each value of the array can be a single file or a directory)
     */
    public void setPropertyFile(File[] files) {
        for (int i = 0; i < files.length; i++)
            setPropertyFile(files[i]);
    }

    // ==================================================================
    //
    // Public methods of BrowserProperty interface.
    //
    // ==================================================================

    /**
     * Fixes the context property to use to constructs the wrappers
     * @param properties The context property to use
     */
    public void setContextProperty(ContextProperty properties) {
        contextProperty_ = properties;
        if (contextProperty_ != null)
            decoder_ = contextProperty_.getDecoder();
    }

    /**
     * Provides the entry to add into the icon property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getIcon(String id) {
        Node node = getNode(id);
        if (node != null) {
            Icon icon = node.getIcon();
            if (icon != null) {
                if (icon.getKind().equals("icon-file")) {
                    IconFactory iconFactory = new DefaultIconFactory();
                    iconFactory.setUrl(icon.getValue());
                    return createEntry(id, iconFactory);
                } else if (icon.getKind().equals("java-class")) {
                    try {
                        Class factory = ClassResolver.resolve(icon.getValue());
                        if (IconFactory.class.isAssignableFrom(factory))
                            return createEntry(id, factory.newInstance());
                        else
                            System.out.println(icon.getValue() + " is not an IconFactory instance !");
                    } catch (ClassNotFoundException e) {
                        System.out.println(
                            icon.getValue() + " : Class not found !");
                    } catch (InstantiationException e) {
                        e.printStackTrace();
                    } catch (IllegalAccessException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
        return null;
    }

    /**
     * Provides the entry to add into the panel property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getPanel(String id) {
        Node node = getNode(id);
        if (node != null) {
            Panel panel = node.getPanel();
            if (panel != null) {
                try {
                    //System.out.println("[getPanel 1] Thread.currentThread().getContextClassLoader() : " + Thread.currentThread().getContextClassLoader());
                    //System.out.println("[getPanel 2] forName with Thread.currentThread().getContextClassLoader()");
                    //System.out.println("Thread courant : " + Thread.currentThread());
                    Class panelClass =
                        ClassResolver.resolve(panel.getJavaClass());
                    if (NodeViewPanel.class.isAssignableFrom(panelClass)) {
                        PanelFactory panelFactory = new DefaultPanelFactory();
                        panelFactory.setClassName(panelClass);
                        return createEntry(id, panelFactory);
                    } else
                        System.out.println(panel.getJavaClass() + " is not a NodeViewPanel instance !");
                } catch (ClassNotFoundException e) {
                    System.out.println(panel.getJavaClass() + " : Class not found !");
                }
            }
        }
        return null;
    }

    /**
     * Provides the entry to add into the menu property context or null.
     * @param id The research key (a class name)
     * @return The entry to add or null
     */
    public Entry getMenu(String id) {
        Node node = getNode(id);
        if (node != null) {
            Menu menu = node.getMenu();
            if (menu != null) {
                ContextContainer items = new DefaultContextContainer();
                ContextContainer childItems = new DefaultContextContainer();
                List itemList = menu.getItemList();
                for (Iterator i = itemList.iterator(); i.hasNext();) {
                    Item item = (Item) i.next();
                    ItemProperty ip = new ItemProperty(item.getLabel(),item.getJavaClass(),item.getTreeChildVisible().equals("false") ? false : true,item.getTypeChildVisible().equals("true") ? true : false);
                    if (item.getTreeChildVisible().equals("false"))
                        items.addEntry(item.getLabel(), ip);
                    else
                        childItems.addEntry(item.getLabel(), ip);
                }
                MenuFactoryConfiguration menuFactory = new DefaultMenuFactory();
                menuFactory.setInheritTreeMenu(menu.getInheritTreeMenu().equals("false") ? false : true);
                menuFactory.setInheritTypeMenu(menu.getInheritTypeMenu().equals("false") ? false : true);
                menuFactory.setItems(items);
                menuFactory.setChildItems(childItems);
                return createEntry(id, menuFactory);
            }
        }
        return null;
    }

    /**
     * Provides the entry to add into the wrapper property context or null.
     * @param id The research key
     * @return The entry to add or null
     */
    public Entry getWrapper(String id) {
        Node node = getNode(id);
        if (node != null) {
            Wrapper wrapper = node.getWrapper();
            if (wrapper != null) {
                try {
                    Class wrapperClass = ClassResolver.resolve(wrapper.getJavaClass());
                    if (org.objectweb.util.browser.naming.api.Wrapper.class.isAssignableFrom(wrapperClass) && Context.class.isAssignableFrom(wrapperClass)) {
                        SpecificContextFactory cf = new DefaultContextFactory(decoder_);
                        cf.setClassName(wrapperClass);
                        return createEntry(id, cf);
                    } else
                        System.out.println(wrapper.getJavaClass() + " is not a Wrapper and a Context instance !");
                } catch (ClassNotFoundException e) {
                    System.out.println(wrapper.getJavaClass() + " : Class not found !");
                }
            }
        }
        return null;
    }

    protected class XMLFileFilter implements java.io.FileFilter {
        public boolean accept(File pathname) {
            String extension = pathname.getName().substring(pathname.getName().lastIndexOf('.') + 1);
            return extension.equalsIgnoreCase("xml") && !pathname.isDirectory();

        }
    }

}
