@echo off
rem # ====================================================================
rem #
rem # OpenCCM: The Open CORBA Component Model Platform
rem # Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
rem # Contact: openccm@objectweb.org
rem #
rem # This library is free software; you can redistribute it and/or
rem # modify it under the terms of the GNU Lesser General Public
rem # License as published by the Free Software Foundation; either
rem # version 2.1 of the License, or any later version.
rem #
rem # This library is distributed in the hope that it will be useful,
rem # but WITHOUT ANY WARRANTY; without even the implied warranty of
rem # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
rem # Lesser General Public License for more details.
rem #
rem # You should have received a copy of the GNU Lesser General Public
rem # License along with this library; if not, write to the Free Software
rem # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
rem # USA
rem #
rem # Initial developer(s): Marc Navarro (mnavarro@wins.uva.nl),
rem #                       Philippe Merle.
rem # Contributor(s): Christophe Demarey, Jerome Offroy.
rem #
rem # ====================================================================
rem #
rem # Starting an OpenCCM Java Component Server.
rem #
rem # ====================================================================

setlocal

rem #  =====================
rem #  Compute Command Line
rem #  =====================

set /a nb_args=0
for %%i in (%*) do set /a nb_args=nb_args+1

if %nb_args% == 0 goto help

set /a nb=0
set with_ots="false"
set with_trace="false"
set services_list=

:initial
if "%1" == "-h" goto help
if "%1" == "--ots" goto ots
if "%1" == "--trace" goto trace
if %nb% == %nb_args% (goto end_command_line) else (goto arguments)

:ots
set with_ots="true"
set services_list=%services_list% Transactionnal
shift
set /a nb=nb+1
goto initial

:trace
set with_trace="true"
set services_list=%services_list% Trace
shift
set /a nb=nb+1
goto initial

:arguments
set command_args=%command_args% %1
set server_name=%1
shift
set /a nb=nb+1
goto initial

:help
echo usage: jcs_start [--ots] [--trace] ^<server_name^>
goto :eof

:end_command_line

set JCS_PATH=%OpenCCM_CONFIG_DIR%\ComponentServers\%server_name%

rem #  =============
rem #  JCS started ?
rem #  =============

rem # Check if this OpenCCM Java Component Server is already started.
if not exist %JCS_PATH%.PID goto :platform_installed

echo Error: The OpenCCM Java Component Server %server_name% is already started!
goto :eof

rem #  ============================
rem #  OpenCCM Platform installed ?
rem #  ============================

:platform_installed
call %OpenCCM_HOMEDIR%\bin\ccm_installed
if %CCM_INSTALLED%==0 goto :eof

rem #  ===========================
rem #  CosNaming Service started ?
rem #  ===========================

rem # Obtain the IOR of the started CosNaming Service.
call ns_ior > %OpenCCM_CONFIG_DIR%\ComponentServers\%server_name%.tmp
for /f %%i in (%OpenCCM_CONFIG_DIR%\ComponentServers\%server_name%.tmp) do set ior_NS=%%i
del %OpenCCM_CONFIG_DIR%\ComponentServers\%server_name%.tmp

rem # Check if a CosNaming Service is started.
if defined ior_NS goto ior_ots

echo Error: The CosNaming Service is not started!
goto :eof

rem #  ===============
rem #  Obtain IOR OTS.
rem #  ===============

:ior_ots

if %with_ots% == "false" goto jcs

rem # Obtain the IOR of the started CosTransactions Service.
call ots_ior > %OpenCCM_CONFIG_DIR%\ComponentServers\%server_name%.tmp
for /f %%i in (%OpenCCM_CONFIG_DIR%\ComponentServers\%server_name%.tmp) do set ior_OTS=%%i
del %OpenCCM_CONFIG_DIR%\ComponentServers\%server_name%.tmp

rem # Check if a CosTransactions Service is started.
if defined ior_OTS goto :jcs

echo Error: The CosTransactions Service is not started!
goto :eof

rem #  ===========================================
rem #  Starting the OpenCCM Java Component Server.
rem #  ===========================================

:jcs
echo The OpenCCM Java Component Server %server_name% will be started with [%services_list%]...
rem # Check if the OpenCCM Java Component Server archive directory doesn't exist.
if exist %JCS_PATH%.archive_cache goto cache_exist

echo Creating the %JCS_PATH%.archive_cache directory.
mkdir %JCS_PATH%.archive_cache

:cache_exist
rem # Check if the OpenCCM Java Component Server specific directory doesn't exist.
if exist %JCS_PATH%\ goto ComponentServer_Directory_exist

echo Creating the %JCS_PATH%\ specific directory.
mkdir %JCS_PATH%\

:ComponentServer_Directory_exist
set jvm_args=
set run_id=jcs
if %with_ots% == "false" goto service_trace

rem # CosTransactions Service
rem #
set jvm_args=-Dior_ots=%ior_OTS%
set run_id=%run_id%_ots

:service_trace
if %with_trace% == "false" goto run
set run_id=%run_id%_trace
rem 
rem Copying the Trace Service configuration file to the specific ComponentServer directory.
copy /Y %JCS_PATH%\..\..\log\monolog.CCM.properties %JCS_PATH%\monolog.CCM.properties > null
del null

:run
echo Launching an OpenCCM Java Component Server.
set jvm_args=%jvm_args% -Djcs_path=%JCS_PATH% -Dior_ns=%ior_NS%
set args=--runid %run_id% %server_name% %JCS_PATH%.IOR

echo Backup the current directory.
call cd > %JCS_PATH%\lastDirectory.tmp
for /f %%i in (%JCS_PATH%\lastDirectory.tmp) do set BACKUP_DIRECTORY=%%i
del %JCS_PATH%\lastDirectory.tmp

rem
rem Launch an OpenCCM Java Component Server from its specific directory
cd %JCS_PATH%\
call launcher fork "%jvm_args%" "%args% %OpenCCM_HOMEDIR%/xml/launcher/JCS.xml" %JCS_PATH%.output %JCS_PATH%.PID
cd %BACKUP_DIRECTORY%

echo The OpenCCM Java Component Server %server_name% is started.
