/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2001-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/

/* TO DO verify|make compatibility with PocketPC */

#include <stdio.h>
#include <process.h>
#include <windows.h>
#include <winbase.h>
#define MAX_COMMAND_SIZE 1000

void Usage();
char **cutStringPath(char * ,int ,char ,int * );
FILE *setOutPID(char *);

/********************************************************************
 * exec.exe                                                         *
 * Usage : exec.exe stdout|stderr|filename cmd [args]               *
 *                                                                  *
 * execute in background a command with arguments in a new Process  *
 *                                        and show or store its PID *
 ********************************************************************/
void main(int argc , char **argv)
{
	char command[MAX_COMMAND_SIZE];         /* stock the command name */
	char *args;								/* stock args */
	char pathDir[1024];						/* String for PATH expansion */
	int nbDirInPath;						/* number of different directories */
	char **pathDirCut;						/* separate Strings representing PATH */
	char expandedCmdName[MAX_COMMAND_SIZE];	/* built command PATH+command */
	int i;
	FILE * out = stdout;		/* filedescriptor to write PID */
	BOOL bRet ;					/* return value for CreateProcess */
	STARTUPINFO sui ;			/* init info for CreateProcess */
	PROCESS_INFORMATION pi ;	/* caracteristic of the newly created process */

	if (argc < 3){ Usage();exit(-1); }	/* Bad parameter nomber */
	out = setOutPID(argv[1]);			/* set the output for PID writting */
		
	/* create command and argument for CreateProcess */
	args = (char *)malloc(sizeof(char*)*(argc)*MAX_COMMAND_SIZE);
	args[0] = '\0';
	i= 3 ;
	while ( i < argc && strstr(argv[i],">") == NULL){
		strcat(args," ");
		strcat(args,argv[i++]);
		strcat(args," ");
	}
	sprintf(command,"%s",argv[2]);
	if (strstr(_strlwr( command),".exe") == NULL ) strcat(command,".exe") ;
	/* create init info for CreateProcess */
	sui.cb = sizeof (STARTUPINFO);
	GetStartupInfo (&sui);
	sui.dwFlags = STARTF_USESHOWWINDOW ;
	sui.wShowWindow = SW_SHOW ;
	
	/* is the command absolute filename */
	if (strchr(command,'\\') != NULL) {
		/* CreateProcess */
		bRet = CreateProcess (	command,					
								args,
								NULL, NULL, TRUE,
								0, NULL, NULL,
								&sui,
								&pi);
	}else {
		ExpandEnvironmentStrings("%PATH%\0",pathDir,1024);
		pathDirCut = cutStringPath(pathDir,1024,';',&nbDirInPath);
		bRet = 0;i = 0 ; 
		while(bRet == 0 && i < nbDirInPath ){
			sprintf(expandedCmdName,"%s\\%s",pathDirCut[i],command); /* create expanded command name */
			/* CreateProcess */
			bRet = CreateProcess (	expandedCmdName,		
									args,
									NULL, NULL, TRUE,
									0, NULL, NULL,
									&sui,
									&pi);
									i++;
		}
	}
	/* print PID or error if no process created */
	if (bRet != 0) {
		fprintf(out,"%u\n",pi.dwProcessId);
		fclose(out);
	}else {
		fprintf(stderr,"command %s is unknown\n",command);
		exit(-3);
	}
	exit( 0 );
}

/********************
 * internal methods *
 ********************/
void Usage(){fprintf(stderr,"Usage : exec stdout|filename prog [args]\n");}

/* separatre fields separated by 'separator' */
char **cutStringPath(char *base,int size,char separator,int *nbFields){
	char c = ' ' , **res,buffer[MAX_COMMAND_SIZE];
	int idRes=0,idBase=0,idBuffer = 0;
	int stop = 0; /* false value */
	int directoryLenght = MAX_COMMAND_SIZE;

	*nbFields = 0;
	c = base[idBase];
	/* compute number of fields separated by separator */
	while (c != '\0'){
		if (c == separator){ (*nbFields)++;}
		idBase++;
		c = base[idBase];
	}
	(*nbFields)++;
	
	/* allocation of the array of fields */
	res = (char **)malloc(sizeof(char *) * (*nbFields));
	for (idRes = 0 ; idRes < *nbFields ; idRes ++){
		res[idRes] = (char *) malloc(sizeof(char) * directoryLenght);	
	}

	/* separate String into Strings */
	idRes = 0;idBase = 0;c= ' ';
	while( c != '\0' && idBase < size ){
		c = base[idBase];
		if (c != separator ) {
			buffer[idBuffer++] = c;
		}else {
			buffer[idBuffer] = '\0';
			sprintf(res[idRes++],"%s",buffer) ;
			idBuffer = 0;
		}
		idBase++;
	}
	buffer[idBuffer] = '\0';
	sprintf(res[idRes],"%s",buffer) ;

	return res;
}

FILE *setOutPID(char *filename){
	FILE * out;
	if (strcmp(filename,"stdout") == 0)     { out = stdout;}
	else if(strcmp(filename,"stderr") == 0) { out = stderr;}
	else {
		out = fopen(filename,"w");

		if (out == NULL) {
			fprintf(stderr,"Can't write file %s\n",filename);
			exit(-2);
		}
	}
	return out;
}