/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher ;

import org.objectweb.util.launcher.parser.Parser;
import org.objectweb.util.launcher.parser.RunDescription;
import org.objectweb.util.launcher.parser.OptionDescription;


/**
 * CommandProvider tool for completing CommandJava objects<BR>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class CommandProvider {

    /**
     * Retrieve the command java identified by name using the parser
     *
     * @param parser the parser to use
     * @param name the name of the run
     *
     * @return the configured java command
     */
    public static CommandJava get(Parser parser, String name) {
        CommandJava cmd = CommandFactory.instance().create();
        cmd.setName(name);
        TraceSystem.get("launcher").info("Completing java command " + cmd.getName());
        RunDescription desc = parser.getRun(cmd.getName());
        cmd.setClassname(desc.getMainclass());
        try {
            cmd.getLoader().addURL(desc.getClasses().toArray());
        } catch (java.net.MalformedURLException ex) {
            throw new LauncherException(ex);
        }
        cmd.addArguments(desc.getArguments());
        cmd.addProperties(desc.getProperties());
        return cmd;
    }
}
