/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package  org.objectweb.util.launcher ;



/**
 * Defines a mode of execution
 *
 * <p>
 *  This mode of execution is a sequential mode which start the 
 *  described application in a different thread than the lancher one.
 *  The daemon thread is stopped when the launcher exit (according to 
 *  the daemon definition of the java.lang.Thread class). 
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class ModeFactory
{
    /**
     * Create a specific instance of a mode described by its type
     *
     * @param type - the type of mode requested (normal, thread, daemon, ...)
     *
     * @return the mode requested (null if not found)
     */
    public static Mode 
    create(String type) {
        if (type.equals("normal"))
            return new ModeNormal();        
        if (type.equals("thread"))
            return new ModeThread();
        if (type.equals("deamon"))
            return new ModeDaemon();
        TraceSystem.get("launcher").warn("No Mode found for "+type);
        return null;
    }
}
