/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher.cmdline ;


import java.util.Vector;

import org.objectweb.util.cmdline.lib.DefaultOptionArgument;
import org.objectweb.util.cmdline.api.Iterator;

import org.objectweb.util.launcher.CommandJava;
import org.objectweb.util.launcher.CommandFactory;
import org.objectweb.util.launcher.XBootClassLoader;
import org.objectweb.util.launcher.StringList;
import org.objectweb.util.launcher.TraceSystem;

import org.objectweb.util.launcher.parser.Parser;
import org.objectweb.util.launcher.parser.RunDescription;
import org.objectweb.util.launcher.parser.OptionDescription;


/**
 * Definition of the --runid option<BR>
 * <p>
 * This option specify the run identifier to use for starting the 
 * application
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class OptionOption
    extends DefaultOptionArgument
    implements OptionLauncher
{
    public final static String shortTag = "-opt" ;
    public final static String longTag = "--option" ;

    protected Parser parser_ ;
    protected Vector options_ ;

    /**
     * Default Constructor
     */
    public OptionOption(Parser p){
        super(new String[]{shortTag, longTag},
              "<OptionID>",
              "The name of an option",
              "");
        parser_=p;
        options_=new Vector();
    }


    protected class OptionBuilder {
        protected String opt ;

        public OptionBuilder(String arg) {
            this.opt = arg ;
        }
        
        /**
         * Complete the command java by setting the identifier of
         * the run to use
         *
         * @param cmd - the command to complete
         */
        public void complete(){
            OptionDescription desc = OptionOption.this.parser_.getOption(opt);
            CommandFactory.instance().getDefaultClasspath().addAll(desc.getClasses());
            CommandFactory.instance().getDefaultArguments().addAll(desc.getArguments());
            CommandFactory.instance().getDefaultProperties().putAll(desc.getProperties());
        }
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator) {
        setArgument(consumeArgument(iterator));
        options_.addElement(new OptionBuilder(getArgument()));
    }
    
    /**
     * Complete the command java by setting the identifier of
     * the run to use
     *
     * @param cmd - the command to complete
     */
    public CommandJava[] complete() {
        java.util.Iterator options = options_.iterator();
        Vector list = new Vector();
        while (options.hasNext())
            ((OptionBuilder)options.next()).complete();           
        return new CommandJava[0];
    }


    public static String create(String opt) {
        return shortTag+" "+opt+" " ; 
    }
}
