/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher.cmdline ;


import java.util.Vector;

import java.util.regex.Pattern ;

import org.objectweb.util.cmdline.lib.DefaultOptionArgument;
import org.objectweb.util.cmdline.api.Iterator;

import org.objectweb.util.launcher.CommandJava;
import org.objectweb.util.launcher.CommandFactory;
import org.objectweb.util.launcher.XBootClassLoader;
import org.objectweb.util.launcher.StringList;
import org.objectweb.util.launcher.TraceSystem;

import org.objectweb.util.launcher.parser.Parser;
import org.objectweb.util.launcher.parser.RunDescription;
import org.objectweb.util.launcher.parser.OptionDescription;


/**
 * Definition of the --runid option<BR>
 * <p>
 * This option specify the run identifier to use for starting the 
 * application
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class OptionProperty
    extends DefaultOptionArgument
//     implements OptionLauncher
{
    public final static String shortTag = "-D" ;
    public final static String longTag = "--prop" ;

    /**
     * Default Constructor
     */
    public OptionProperty(){
        super(new String[]{shortTag, longTag},
              "<name=value>",
              "The property to set",
              "");
    }

    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator) {
        setArgument(consumeArgument(iterator));
        String[] p = Pattern.compile("=").split(getArgument());
        if (p.length==2)
            CommandFactory.instance().getDefaultProperties().put(p[0],p[1]);
        else if (p.length==1)
            CommandFactory.instance().getDefaultProperties().put(p[0],"");
    }

    public static String create(String name, String value) {
        return shortTag+" "+name+"="+value+" " ; 
    }
}
