/*====================================================================

ObjectWeb Util Launcher Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Romain Rouvoy.
Contributor(s): .

====================================================================*/

package org.objectweb.util.launcher.cmdline ;


import java.util.Vector;

import org.objectweb.util.cmdline.lib.DefaultOptionArgument;
import org.objectweb.util.cmdline.api.Iterator;

import org.objectweb.util.launcher.CommandJava;
import org.objectweb.util.launcher.CommandFactory;
import org.objectweb.util.launcher.XBootClassLoader;
import org.objectweb.util.launcher.StringList;
import org.objectweb.util.launcher.TraceSystem;

import org.objectweb.util.launcher.parser.Parser;
import org.objectweb.util.launcher.parser.RunDescription;
import org.objectweb.util.launcher.parser.OptionDescription;


/**
 * Definition of the --runid option<BR>
 * <p>
 * This option specify the run identifier to use for starting the 
 * application
 * </p>
 *
 * @author <a href="mailto:Romain.Rouvoy@lifl.fr">Romain Rouvoy</a>
 *
 * @version 0.1
 */
public class OptionRun
    extends DefaultOptionArgument
    implements OptionLauncher
{
    public final static String shortTag = "-run" ;
    public final static String longTag = "--runid" ;

    protected Parser parser_ ;
    protected Vector runs_ ;

    /**
     * Default Constructor
     */
    public OptionRun(Parser p){
        super(new String[]{shortTag, longTag},
              "<RunID>",
              "The identifier of the run to launch",
              "default");
        parser_=p;
        runs_ = new Vector();
    }

    
    protected class RunBuilder {
        protected String argument_ ;

        public RunBuilder(String arg) {
            this.argument_ = arg ;
        }
        
        protected void completeRun(CommandJava cmd,String run) {
            cmd.setName(run);
            TraceSystem.get("launcher").info("Completing java command " + cmd.getName());
            RunDescription desc = OptionRun.this.parser_.getRun(cmd.getName());
            cmd.setClassname(desc.getMainclass());
            try {
                cmd.getLoader().addURL(desc.getClasses().toArray());
            } catch (java.net.MalformedURLException ex) {
                TraceSystem.get("launcher").error("Exception raised: " + ex);
            }
            cmd.addArguments(desc.getArguments());
            cmd.addProperties(desc.getProperties());
        }
        

        /**
         * Complete the command java by setting the identifier of
         * the run to use
         *
         * @param cmd - the command to complete
         */
        public CommandJava complete(){
            CommandJava cmd = CommandFactory.instance().create();
            completeRun(cmd,argument_);
            return cmd;
        }
    }


    // ==================================================================
    //
    // Public methods for interface org.objectweb.util.cmdline.api.Option
    //
    // ==================================================================

    /**
     * Consumes command line arguments from an iterator.
     *
     * @param iterator The command line argument iterator.
     */
    public void
    consume(org.objectweb.util.cmdline.api.Iterator iterator)
    {
        setArgument(consumeArgument(iterator));
        runs_.addElement(new RunBuilder(getArgument()));
    }


    
    /**
     * Complete the command java by setting the identifier of
     * the run to use
     *
     * @param cmd - the command to complete
     */
    public CommandJava[] complete() {
//         if (runs_.size() == 0) {
//             CommandJava[] cmd = new CommandJava[1];
//             cmd[0] = (new RunBuilder(getArgument())).complete();
//             return cmd;                
//         }else{            
        java.util.Iterator runs = runs_.iterator();
        Vector list = new Vector();
        while (runs.hasNext())
            list.addElement(((RunBuilder)runs.next()).complete());           
        return (CommandJava[])list.toArray(new CommandJava[0]);
//         }
    }


    public static String create(String run) {
        return shortTag+" "+run+" " ; 
    }
}
