/**
 * copyright 2002 2003 Laboratoire d'Informatique Paris 6 (LIP6)
 * 
 * This file is part of ModFact.
 * 
 * ModFact is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option) any later version.
 * 
 * ModFact is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Foobar; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.lip6.src.modfact.service;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;

import fr.lip6.src.modfact.generation.dtd.GenerationXMIDTD1_1;
import fr.lip6.src.modfact.generation.idl.GenerationMOFIDL1_3;
import fr.lip6.src.modfact.generation.idl2javaImpl
	.GenerationIDL2JavaImplMOF1_1;
import fr.lip6.src.modfact.generation.jmi.GenerationJMI1_0;

import fr.lip6.src.modfact.io.Export;
import fr.lip6.src.modfact.io.Import;
import fr.lip6.src.modfact.util.Helper;

public class Client {
	boolean idl = false;
	boolean java = false;
	boolean jmi = false;
	boolean dtd = false;
	String input = null;

	static org.omg.mof.Model.ModelPackageFactory _reference;
	public static void main(String[] args) {
		try {
			Client client = new Client();
			client.getParameters(args);

			org.omg.mof.Model.Package _package = null;

			// create and initialize the ORB
			ORB orb = ORB.init(args, null);

			// get the root naming context
			org.omg.CORBA.Object objRef =
				orb.resolve_initial_references("NameService");
			// Use NamingContextExt instead of NamingContext. This is 
			// part of the Interoperable naming Service.  
			NamingContextExt ncRef = NamingContextExtHelper.narrow(objRef);

			// resolve the Object Reference in Naming
			String name = "MDAFactoryServer";
			_reference =
				org.omg.mof.Model.ModelPackageFactoryHelper.narrow(
					ncRef.resolve_str(name));

			//creation of Model 
			org.omg.mof.Model.ModelPackage _model_package =
				_reference.create_model_package();

			//initialisation  
			Import imp = new Import();
			imp.initiate(_model_package, orb);

			//input XML model
			_package = imp.parse(client.input);
			System.out.println(
				"Package :" + _package.name() + " is in the repository");

			//Generation of IDL
			if (client.idl) {
				Logger logger =
					Logger.getLogger(
						"fr.lip6.src.mdafactory.generation.idl.GenerationMOFIDL1_3");
				logger.setLevel(Level.ALL);
				GenerationMOFIDL1_3 generator = new GenerationMOFIDL1_3();
				generator.setOut("." + File.separator + "out.idl");
				generator.generateIDL(_package);
			}

			//Generation of DTD
			if (client.dtd) {
				Logger logger =
					Logger.getLogger(
						"fr.lip6.src.mdafactory.generation.dtd.GenerationXMIDTD1_1");
				logger.setLevel(Level.ALL);
				GenerationXMIDTD1_1 generator = new GenerationXMIDTD1_1();
				generator.setOut("." + File.separator + "out.dtd");
				generator.generateDTD(_package);
			}

			//Generation of code
			if (client.java) {
				Logger logger =
					Logger.getLogger(
						"fr.lip6.src.mdafactory.generation.idl2javaImpl.GenerationIDL2JavaImplMOF1_1");
				logger.setLevel(Level.ALL);
				GenerationIDL2JavaImplMOF1_1 generator =
					new GenerationIDL2JavaImplMOF1_1();
				generator.setDirectory("."+File.separator);
				generator.generateJava(_package);
			}

			//Generation of JMI
			if (client.jmi) {
				Logger logger =
					Logger.getLogger(
						"fr.lip6.src.mdafactory.generation.jmi.GenerationJMI1_0");
				logger.setLevel(Level.ALL);
				GenerationJMI1_0 generator = new GenerationJMI1_0();
				generator.setDirectory("."+File.separator);
				generator.generateJMI1_0(_package);
			}

		} catch (Exception e) {
			System.out.println("ERROR : " + e);
			e.printStackTrace(System.out);
		}
	}

	public void getParameters(String[] args) {
		for (int i = 0; i < args.length; i++) {
			if ("-idl".compareTo(args[i]) == 0)
				idl = true;
			else if ("-java".compareTo(args[i]) == 0)
				java = true;
			else if ("-dtd".compareTo(args[i]) == 0)
				dtd = true;
			else if ("-jmi".compareTo(args[i]) == 0)
				jmi = true;
			else if (("-input".compareTo(args[i]) == 0) && i + 1 < args.length)
				input = args[++i];
		}
		if ((idl || java || dtd || jmi) && input != null) {
		} else {
			printUsage();
			System.exit(0);
		}
	}

	public void printUsage() {
		System.out.println(
			"Usage: fr.lip6.src.mdafactory.service.Client [-idl] [-java] [-dtd] [-jmi] -input File");
		System.out.println("\t-idl: generation of IDL");
		System.out.println("\t-java: generation of Java");
		System.out.println("\t-dtd: generation of DTD");
		System.out.println("\t-jmi: generation of JMI");
	}
}
